@@ -1,17 +1,18 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef __AFSD_H_ENV__
-#define __AFSD_H_ENV__ 1
+#ifndef OPENAFS_WINNT_AFSD_AFSD_H
+#define OPENAFS_WINNT_AFSD_AFSD_H 1
 
 #define USE_BPLUS 1
 
+#include <afsconfig.h>
 #include <afs/param.h>
 
 BOOL InitClass(HANDLE);
@@ -28,6 +29,8 @@ BOOL APIENTRY About(HWND, unsigned int, unsigned int, long);
 #include <osi.h>
 #include <afs/vldbint.h>
 #include <afs/afsint.h>
+#define FSINT_COMMON_XG
+
 #include <afs/prs_fs.h>
 
 #include "cm_config.h"
@@ -38,9 +41,9 @@ BOOL APIENTRY About(HWND, unsigned int, unsigned int, long);
 #include "cm_diskcache95.h"
 #endif /* DISKCACHE95 */
 #include "cm_conn.h"
+#include "cm_cell.h"
 #include "cm_aclent.h"
 #include "cm_server.h"
-#include "cm_cell.h"
 #include "cm_volstat.h"
 #include "cm_volume.h"
 #include "cm_dcache.h"
@@ -51,13 +54,13 @@ BOOL APIENTRY About(HWND, unsigned int, unsigned int, long);
 #include "cm_btree.h"
 #include "cm_daemon.h"
 #include "cm_ioctl.h"
+#include "smb_iocons.h"
 #include "cm_dnlc.h"
 #include "cm_buf.h"
 #include "cm_memmap.h"
 #include "cm_freelance.h"
 #include "cm_performance.h"
-#include "smb_ioctl.h"
-#include "smb_iocons.h"
+#include "cm_rdr.h"
 #include "afsd_init.h"
 #include "afsd_eventlog.h"
 
@@ -99,10 +102,6 @@ extern BOOL reportSessionStartups;
 #ifdef AFS_FREELANCE_CLIENT
 extern char *cm_FakeRootDir;				// the fake root.afs directory
 
-extern int cm_noLocalMountPoints;			// no. of fake mountpoints
-
-extern cm_localMountPoint_t* cm_localMountPoints;	// array of fake mountpoints
-
 extern int cm_fakeDirSize;				// size (in bytes) of fake root.afs directory
 
 extern int cm_fakeDirCallback;				// state of the fake root.afs directory. indicates
@@ -113,12 +112,20 @@ extern int cm_fakeGettingCallback;			// 1 if currently updating the fake root.af
 #endif /* AFS_FREELANCE_CLIENT */
 
 extern int cm_dnsEnabled;
-extern int cm_freelanceEnabled;
+extern int cm_readonlyVolumeVersioning;
 
 extern long rx_mtu;
 
 extern HANDLE WaitToTerminate;
 
+extern int RDR_Initialized;
+
+extern afs_uint32 smb_Enabled;
+
+extern int cm_virtualCache;
+
+extern afs_int32 cm_verifyData;
+
 #define DFS_SUPPORT 1
 #define LOG_PACKET 1
 #undef  NOTSERVICE
@@ -143,4 +150,4 @@ typedef BOOL ( APIENTRY * AfsdStoppingHook )(void);
 typedef BOOL ( APIENTRY * AfsdStoppedHook )(void);
 
 #define SERVICE_CONTROL_CUSTOM_DUMP 128
-#endif /* AFSD_H_ENV */
+#endif /* OPENAFS_WINNT_AFSD_AFSD_H */
