@@ -50,43 +50,42 @@ int afs_fakestat_enable = 0;	/* 1: fakestat-all, 2: fakestat-crosscell */
  */
 static int
 EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
-              struct volume **avolpp, register struct vrequest *areq,
-	      afs_uint32 *acellidxp, afs_uint32 *avolnump, afs_uint32 *avnoidp)
+              struct volume **avolpp, struct vrequest *areq,
+	      afs_uint32 *acellidxp, afs_uint32 *avolnump,
+	      afs_uint32 *avnoidp, afs_uint32 *auniqp)
 {
     struct volume *tvp = 0;
     struct VenusFid tfid;
     struct cell *tcell;
-    char *cpos, *volnamep, *x;
-    char *buf;
+    char *cpos, *volnamep = NULL;
+    char *buf, *endptr;
     afs_int32 prefetch;		/* 1=>None  2=>RO  3=>BK */
     afs_int32 mtptCell, assocCell = 0, hac = 0;
     afs_int32 samecell, roname, len;
-    afs_uint32 volid, cellidx, vnoid = 0;
+    afs_uint32 volid = 0, cellidx, vnoid = 0, uniq = 0;
 
+    /* Start by figuring out and finding the cell */
     cpos = afs_strchr(data, ':');	/* if cell name present */
     if (cpos) {
-	cellnum = 0;
 	volnamep = cpos + 1;
 	*cpos = 0;
-	for (x = data; *x >= '0' && *x <= '9'; x++)
-	    cellnum = (cellnum * 10) + (*x - '0');
-	if (cellnum && !*x)
+	if ((afs_strtoi_r(data, &endptr, &cellnum) == 0) &&
+	    (endptr == cpos))
 	    tcell = afs_GetCell(cellnum, READ_LOCK);
 	else {
 	    tcell = afs_GetCellByName(data, READ_LOCK);
-	               cellnum = 0;
+	    cellnum = 0;
 	}
 	*cpos = ':';
     } else if (cellnum) {
 	volnamep = data;
 	tcell = afs_GetCell(cellnum, READ_LOCK);
     } else {
-	/*printf("No cellname %s , or cellnum %d , returning ENODEV\n", 
-	       data, cellnum);*/
+	/* No cellname or cellnum; return ENODEV */
 	return ENODEV;
     }
     if (!tcell) {
-	/*printf("Lookup failed, returning ENODEV\n");*/
+	/* no cell found; return ENODEV */
 	return ENODEV;
     }
 
@@ -98,22 +97,38 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
     }
     afs_PutCell(tcell, READ_LOCK);
 
-    cpos = afs_strrchr(volnamep, ':'); /* if vno present */
-    if (cpos) 
+    /* If there's nothing to look up, we can't proceed */
+    if (!*volnamep)
+	return ENODEV;
+
+    /* cell found. figure out volume */
+    cpos = afs_strchr(volnamep, ':');
+    if (cpos)
 	*cpos = 0;
+
     /* Look for an all-numeric volume ID */
-    volid = 0;
-    for (x = volnamep; *x >= '0' && *x <= '9'; x++)
-	volid = (volid * 10) + (*x - '0');
-    if (cpos) {
-	*cpos = ':';
-	vnoid = 0;
-	if (!*x) /* allow vno with numeric volid only */
-	    for (x = (cpos + 1); *x >= '0' && *x <= '9'; x++)
-		vnoid = (vnoid * 10) + (*x - '0');
-	if (*x)
-	    vnoid = 0;
-    }
+    if ((afs_strtoi_r(volnamep, &endptr, &volid) == 0) &&
+	((endptr == cpos) || (!*endptr)))
+    {
+	/* Ok. Is there a vnode and uniq? */
+	if (cpos) {
+	    char *vnodep = (char *)(cpos + 1);
+	    char *uniqp = NULL;
+	    if ((!*vnodep) /* no vnode after colon */
+		|| !(uniqp = afs_strchr(vnodep, ':')) /* no colon for uniq */
+		|| (!*(++uniqp)) /* no uniq after colon */
+		|| (afs_strtoi_r(vnodep, &endptr, &vnoid) != 0) /* bad vno */
+		|| (*endptr != ':') /* bad vnode field */
+		|| (afs_strtoi_r(uniqp, &endptr, &uniq) != 0) /* bad uniq */
+		|| (*endptr)) /* anything after uniq */
+	    {
+		*cpos = ':';
+		/* sorry. vnode and uniq, or nothing */
+		return ENODEV;
+	    }
+	}
+    } else
+	    volid = 0;
 
     /*
      * If the volume ID was all-numeric, and they didn't ask for a
@@ -121,14 +136,10 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
      * as-is.  This is currently only used for handling name lookups
      * in the dynamic mount directory.
      */
-    if (!*x && !avolpp) {
-	if (acellidxp)
-	    *acellidxp = cellidx;
-	if (avolnump)
-	    *avolnump = volid;
-	if (avnoidp)
-	    *avnoidp = vnoid;
-	return 0;
+    if (volid && !avolpp) {
+	if (cpos)
+	    *cpos = ':';
+	goto done;
     }
 
     /*
@@ -137,14 +148,14 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
      * and don't second-guess them by forcing use of a RW volume when
      * they gave the ID of something else.
      */
-    if (!*x && type == '%') {
+    if (volid && type == '%') {
 	tfid.Fid.Volume = volid;	/* remember BK volume */
 	tfid.Cell = mtptCell;
 	tvp = afs_GetVolume(&tfid, areq, WRITE_LOCK);	/* get the new one */
-	if (!tvp) {
-	    /*printf("afs_GetVolume failed - returning ENODEV");*/
-	    return ENODEV;	/* oops, can't do it */
-	}
+	if (cpos) /* one way or another we're done */
+	    *cpos = ':';
+	if (!tvp)
+	    return ENODEV; /* afs_GetVolume failed; return ENODEV */
 	goto done;
     }
 
@@ -178,8 +189,8 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
     /* Get the volume struct. Unless this volume name has ".readonly" or
      * ".backup" in it, this will get the volume struct for the RW volume.
      * The RO volume will be prefetched if requested (but not returned).
+     * Set up to use volname first.
      */
-    /*printf("Calling GetVolumeByName\n");*/
     tvp = afs_GetVolumeByName(volnamep, mtptCell, prefetch, areq, WRITE_LOCK);
 
     /* If no volume was found in this cell, try the associated linked cell */
@@ -207,11 +218,13 @@ EvalMountData(char type, char *data, afs_uint32 states, afs_uint32 cellnum,
 	}
 	osi_FreeSmallSpace(buf);
     }
-
-    if (!tvp) {
-	/*printf("Couldn't find the volume\n");*/
+    /* done with volname */
+    if (cpos)
+	*cpos = ':';
+    if (!tvp)
 	return ENODEV;		/* Couldn't find the volume */
-    }
+    else
+	volid = tvp->volume;
 
     /* Don't cross mountpoint from a BK to a BK volume */
     if ((states & CBackup) && (tvp->states & VBackup)) {
@@ -245,22 +258,24 @@ done:
     if (acellidxp)
 	*acellidxp = cellidx;
     if (avolnump)
-	*avolnump = tvp->volume;
+	*avolnump = volid;
     if (avnoidp)
 	*avnoidp = vnoid;
+    if (auniqp)
+	*auniqp = uniq;
     if (avolpp)
 	*avolpp = tvp;
-    else
+    else if (tvp)
 	afs_PutVolume(tvp, WRITE_LOCK);
     return 0;
 }
 
 int
-EvalMountPoint(register struct vcache *avc, struct vcache *advc,
-	       struct volume **avolpp, register struct vrequest *areq)
+EvalMountPoint(struct vcache *avc, struct vcache *advc,
+	       struct volume **avolpp, struct vrequest *areq)
 {
     afs_int32 code;
-    afs_uint32 avnoid;
+    afs_uint32 avnoid, auniq;
 
     AFS_STATCNT(EvalMountPoint);
 #ifdef notdef
@@ -275,19 +290,22 @@ EvalMountPoint(register struct vcache *avc, struct vcache *advc,
     /* Determine which cell and volume the mointpoint goes to */
     code = EvalMountData(avc->linkData[0], avc->linkData + 1,
                          avc->f.states, avc->f.fid.Cell, avolpp, areq, 0, 0,
-			 &avnoid);
+			 &avnoid, &auniq);
     if (code) return code;
 
     if (!avnoid)
 	avnoid = 1;
 
+    if (!auniq)
+	auniq = 1;
+
     if (avc->mvid == 0)
 	avc->mvid =
 	    (struct VenusFid *)osi_AllocSmallSpace(sizeof(struct VenusFid));
     avc->mvid->Cell = (*avolpp)->cell;
     avc->mvid->Fid.Volume = (*avolpp)->volume;
     avc->mvid->Fid.Vnode = avnoid;
-    avc->mvid->Fid.Unique = 1;
+    avc->mvid->Fid.Unique = auniq;
     avc->f.states |= CMValid;
 
     /* Used to: if the mount point is stored within a backup volume,
@@ -359,11 +377,12 @@ afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
     if (tvc->mvstat != 1)
 	return 0;
 
-    /* Is the call to VerifyVCache really necessary? */
-    code = afs_VerifyVCache(tvc, areq);
-    if (code)
-	goto done;
     if (canblock) {
+	/* Is the call to VerifyVCache really necessary? */
+	code = afs_VerifyVCache(tvc, areq);
+	if (code)
+	    goto done;
+
 	ObtainWriteLock(&tvc->lock, 599);
 	code = EvalMountPoint(tvc, NULL, &tvolp, areq);
 	ReleaseWriteLock(&tvc->lock);
@@ -399,7 +418,7 @@ afs_EvalFakeStat_int(struct vcache **avcp, struct afs_fakestat_state *state,
 #ifdef AFS_DARWIN80_ENV
         root_vp->f.m.Type = VDIR;
         AFS_GUNLOCK();
-        code = afs_darwin_finalizevnode(root_vp, NULL, NULL, 0);
+        code = afs_darwin_finalizevnode(root_vp, NULL, NULL, 0, 0);
         AFS_GLOCK();
         if (code) goto done;
         vnode_ref(AFSTOV(root_vp));
@@ -484,9 +503,9 @@ afs_PutFakeStat(struct afs_fakestat_state *state)
 }
 
 int
-afs_ENameOK(register char *aname)
+afs_ENameOK(char *aname)
 {
-    register int tlen;
+    int tlen;
 
     AFS_STATCNT(ENameOK);
     tlen = strlen(aname);
@@ -496,11 +515,11 @@ afs_ENameOK(register char *aname)
 }
 
 static int
-afs_getsysname(register struct vrequest *areq, register struct vcache *adp,
-	       register char *bufp, int *num, char **sysnamelist[])
+afs_getsysname(struct vrequest *areq, struct vcache *adp,
+	       char *bufp, int *num, char **sysnamelist[])
 {
-    register struct unixuser *au;
-    register afs_int32 error;
+    struct unixuser *au;
+    afs_int32 error;
 
     AFS_STATCNT(getsysname);
 
@@ -509,25 +528,25 @@ afs_getsysname(register struct vrequest *areq, register struct vcache *adp,
     if (!afs_nfsexporter)
 	strcpy(bufp, (*sysnamelist)[0]);
     else {
-	au = afs_GetUser(areq->uid, adp->f.fid.Cell, 0);
+	au = afs_GetUser(areq->uid, adp->f.fid.Cell, READ_LOCK);
 	if (au->exporter) {
 	    error = EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, num, 0);
 	    if (error) {
 		strcpy(bufp, "@sys");
-		afs_PutUser(au, 0);
+		afs_PutUser(au, READ_LOCK);
 		return -1;
 	    } else {
 		strcpy(bufp, (*sysnamelist)[0]);
 	    }
 	} else
 	    strcpy(bufp, afs_sysname);
-	afs_PutUser(au, 0);
+	afs_PutUser(au, READ_LOCK);
     }
     return 0;
 }
 
 void
-Check_AtSys(register struct vcache *avc, const char *aname,
+Check_AtSys(struct vcache *avc, const char *aname,
 	    struct sysname_info *state, struct vrequest *areq)
 {
     int num = 0;
@@ -548,7 +567,7 @@ Check_AtSys(register struct vcache *avc, const char *aname,
 }
 
 int
-Next_AtSys(register struct vcache *avc, struct vrequest *areq,
+Next_AtSys(struct vcache *avc, struct vrequest *areq,
 	   struct sysname_info *state)
 {
     int num = afs_sysnamecount;
@@ -559,7 +578,7 @@ Next_AtSys(register struct vcache *avc, struct vrequest *areq,
 
     /* Check for the initial state of aname != "@sys" in Check_AtSys */
     if (state->offset == -1 && state->allocked == 0) {
-	register char *tname;
+	char *tname;
 
 	/* Check for .*@sys */
 	for (tname = state->name; *tname; tname++)
@@ -579,22 +598,22 @@ Next_AtSys(register struct vcache *avc, struct vrequest *areq,
 	} else
 	    return 0;		/* .*@sys doesn't match either */
     } else {
-	register struct unixuser *au;
-	register afs_int32 error;
+	struct unixuser *au;
+	afs_int32 error;
 
 	*sysnamelist = afs_sysnamelist;
 
 	if (afs_nfsexporter) {
-	    au = afs_GetUser(areq->uid, avc->f.fid.Cell, 0);
+	    au = afs_GetUser(areq->uid, avc->f.fid.Cell, READ_LOCK);
 	    if (au->exporter) {
 		error =
 		    EXP_SYSNAME(au->exporter, (char *)0, sysnamelist, &num, 0);
 		if (error) {
-		    afs_PutUser(au, 0);
+		    afs_PutUser(au, READ_LOCK);
 		    return 0;
 		}
 	    }
-	    afs_PutUser(au, 0);
+	    afs_PutUser(au, READ_LOCK);
 	}
 	if (++(state->index) >= num || !(*sysnamelist)[(unsigned int)state->index])
 	    return 0;		/* end of list */
@@ -628,6 +647,10 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 {
     int nentries;		/* # of entries to prefetch */
     int nskip;			/* # of slots in the LRU queue to skip */
+#ifdef AFS_DARWIN80_ENV
+    int npasses = 0;
+    struct vnode *lruvp;
+#endif
     struct vcache *lruvcp;	/* vcache ptr of our goal pos in LRU queue */
     struct dcache *dcp;		/* chunk containing the dir block */
     afs_size_t temp;		/* temp for holding chunk length, &c. */
@@ -647,9 +670,11 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     long startTime;		/* time we started the call,
 				 * for callback expiration base
 				 */
+    int ftype[4] = {VNON, VREG, VDIR, VLNK}; /* verify type is as expected */
     afs_size_t statSeqNo = 0;	/* Valued of file size to detect races */
     int code;			/* error code */
     long newIndex;		/* new index in the dir */
+    struct DirBuffer entry;	/* Buffer for dir manipulation */
     struct DirEntry *dirEntryp;	/* dir entry we are examining */
     int i;
     struct VenusFid afid;	/* file ID we are using now */
@@ -659,14 +684,12 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     struct volume *volp = 0;	/* volume ptr */
     struct VenusFid dotdot = {0, {0, 0, 0}};
     int flagIndex = 0;		/* First file with bulk fetch flag set */
-    int inlinebulk = 0;		/* Did we use InlineBulk RPC or not? */
+    struct rx_connection *rxconn;
     XSTATS_DECLS;
     dotdot.Cell = 0;
     dotdot.Fid.Unique = 0;
     dotdot.Fid.Vnode = 0;
-#ifdef AFS_DARWIN80_ENV
-    panic("bulkstatus doesn't work on AFS_DARWIN80_ENV. don't call it");
-#endif
+
     /* first compute some basic parameters.  We dont want to prefetch more
      * than a fraction of the cache in any given call, and we want to preserve
      * a portion of the LRU queue in any event, so as to avoid thrashing
@@ -750,6 +773,9 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
      * and it is safe to set the status information for this file.
      */
     statSeqNo = bulkStatCounter++;
+    /* ensure against wrapping */
+    if (statSeqNo == 0)
+	statSeqNo = bulkStatCounter++;
 
     /* now we have dir data in the cache, so scan the dir page */
     fidIndex = 0;
@@ -766,14 +792,14 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	dirCookie = newIndex << 5;
 
 	/* get a ptr to the dir entry */
-	dirEntryp =
-	    (struct DirEntry *)afs_dir_GetBlob(dcp, newIndex);
-	if (!dirEntryp)
+	code = afs_dir_GetBlob(dcp, newIndex, &entry);
+	if (code)
 	    break;
+	dirEntryp = (struct DirEntry *)entry.data;
 
 	/* dont copy more than we have room for */
 	if (fidIndex >= nentries) {
-	    DRelease(dirEntryp, 0);
+	    DRelease(&entry, 0);
 	    break;
 	}
 
@@ -799,49 +825,56 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 		}
 	    } while (tvcp && retry);
 	    if (!tvcp) {	/* otherwise, create manually */
-		tvcp = afs_NewVCache(&tfid, hostp);
+		tvcp = afs_NewBulkVCache(&tfid, hostp, statSeqNo);
 		if (tvcp)
 		{
-			ObtainWriteLock(&tvcp->lock, 505);
-			ReleaseWriteLock(&afs_xvcache);
-			afs_RemoveVCB(&tfid);
-			ReleaseWriteLock(&tvcp->lock);
+		    ObtainWriteLock(&tvcp->lock, 505);
+#ifdef AFS_DARWIN80_ENV
+		    /* use even/odd hack to guess file versus dir.
+		       let links be reaped. oh well. */
+		    if (dirEntryp->fid.vnode & 1)
+			tvcp->f.m.Type = VDIR;
+		    else
+			tvcp->f.m.Type = VREG;
+		    /* finalize to a best guess */
+		    afs_darwin_finalizevnode(tvcp, VTOAFS(adp), NULL, 0, 1);
+		    /* re-acquire usecount that finalizevnode disposed of */
+		    vnode_ref(AFSTOV(tvcp));
+#endif
+		    ReleaseWriteLock(&afs_xvcache);
+		    afs_RemoveVCB(&tfid);
+		    ReleaseWriteLock(&tvcp->lock);
 		} else {
-			ReleaseWriteLock(&afs_xvcache);
+		    ReleaseWriteLock(&afs_xvcache);
 		}
 	    } else {
 		ReleaseWriteLock(&afs_xvcache);
 	    }
 	    if (!tvcp)
 	    {
-		DRelease(dirEntryp, 0);
+		DRelease(&entry, 0);
 		ReleaseReadLock(&dcp->lock);
 		ReleaseReadLock(&adp->lock);
 		afs_PutDCache(dcp);
 		goto done;	/* can happen if afs_NewVCache fails */
 	    }
 
-#ifdef AFS_DARWIN80_ENV
-            if (tvcp->f.states & CVInit) {
-                 /* XXX don't have status yet, so creating the vnode is
-                    not yet useful. we would get CDeadVnode set, and the
-                    upcoming PutVCache will cause the vcache to be flushed &
-                    freed, which in turn means the bulkstatus results won't 
-                    be used */
-            }
-#endif
 	    /* WARNING: afs_DoBulkStat uses the Length field to store a
 	     * sequence number for each bulk status request. Under no
 	     * circumstances should afs_DoBulkStat store a sequence number
 	     * if the new length will be ignored when afs_ProcessFS is
 	     * called with new stats. */
 #ifdef AFS_SGI_ENV
-	    if (!(tvcp->f.states & (CStatd | CBulkFetching))
+	    if (!(tvcp->f.states & CStatd)
+		&& (!((tvcp->f.states & CBulkFetching) &&
+		      (tvcp->f.m.Length != statSeqNo)))
 		&& (tvcp->execsOrWriters <= 0)
 		&& !afs_DirtyPages(tvcp)
 		&& !AFS_VN_MAPPED((vnode_t *) tvcp))
 #else
-	    if (!(tvcp->f.states & (CStatd | CBulkFetching))
+	    if (!(tvcp->f.states & CStatd)
+		&& (!((tvcp->f.states & CBulkFetching) &&
+		      (tvcp->f.m.Length != statSeqNo)))
 		&& (tvcp->execsOrWriters <= 0)
 		&& !afs_DirtyPages(tvcp))
 #endif
@@ -855,16 +888,19 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 		 * CBulkFetching state bit and the value in the file size.
 		 * It is safe to set the status only if the CBulkFetching
 		 * flag is still set and the value in the file size does
-		 * not change.
+		 * not change. NewBulkVCache sets us up for the new ones.
+		 * Set up the rest here.
 		 *
 		 * Don't fetch status for dirty files. We need to
 		 * preserve the value of the file size. We could
 		 * flush the pages, but it wouldn't be worthwhile.
 		 */
+		if (!(tvcp->f.states & CBulkFetching)) {
+		    tvcp->f.states |= CBulkFetching;
+		    tvcp->f.m.Length = statSeqNo;
+		}
 		memcpy((char *)(fidsp + fidIndex), (char *)&tfid.Fid,
 		       sizeof(*fidsp));
-		tvcp->f.states |= CBulkFetching;
-		tvcp->f.m.Length = statSeqNo;
 		fidIndex++;
 	    }
 	    afs_PutVCache(tvcp);
@@ -875,7 +911,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	 * used by this dir entry.
 	 */
 	temp = afs_dir_NameBlobs(dirEntryp->name) << 5;
-	DRelease(dirEntryp, 0);
+	DRelease(&entry, 0);
 	if (temp <= 0)
 	    break;
 	dirCookie += temp;
@@ -904,36 +940,72 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	/* start the timer; callback expirations are relative to this */
 	startTime = osi_Time();
 
-	tcp = afs_Conn(&adp->f.fid, areqp, SHARED_LOCK);
+	tcp = afs_Conn(&adp->f.fid, areqp, SHARED_LOCK, &rxconn);
 	if (tcp) {
-	    hostp = tcp->srvr->server;
-	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_BULKSTATUS);
-	    RX_AFS_GUNLOCK();
+	    hostp = tcp->parent->srvr->server;
 
-	    if (!(tcp->srvr->server->flags & SNO_INLINEBULK)) {
+	    for (i = 0; i < fidIndex; i++) {
+		/* we must set tvcp->callback before the BulkStatus call, so
+		 * we can detect concurrent InitCallBackState's */
+
+		afid.Cell = adp->f.fid.Cell;
+		afid.Fid.Volume = adp->f.fid.Fid.Volume;
+		afid.Fid.Vnode = fidsp[i].Vnode;
+		afid.Fid.Unique = fidsp[i].Unique;
+
+		do {
+		    retry = 0;
+		    ObtainReadLock(&afs_xvcache);
+		    tvcp = afs_FindVCache(&afid, &retry, 0 /* !stats&!lru */);
+		    ReleaseReadLock(&afs_xvcache);
+		} while (tvcp && retry);
+
+		if (!tvcp) {
+		    continue;
+		}
+
+		if ((tvcp->f.states & CBulkFetching) &&
+		     (tvcp->f.m.Length == statSeqNo)) {
+		    tvcp->callback = hostp;
+		}
+
+		afs_PutVCache(tvcp);
+		tvcp = NULL;
+	    }
+
+	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_BULKSTATUS);
+
+	    if (!(tcp->parent->srvr->server->flags & SNO_INLINEBULK)) {
+		RX_AFS_GUNLOCK();
 		code =
-		    RXAFS_InlineBulkStatus(tcp->id, &fidParm, &statParm,
+		    RXAFS_InlineBulkStatus(rxconn, &fidParm, &statParm,
 					   &cbParm, &volSync);
+		RX_AFS_GLOCK();
 		if (code == RXGEN_OPCODE) {
-		    tcp->srvr->server->flags |= SNO_INLINEBULK;
-		    inlinebulk = 0;
+		    tcp->parent->srvr->server->flags |= SNO_INLINEBULK;
+		    RX_AFS_GUNLOCK();
 		    code =
-			RXAFS_BulkStatus(tcp->id, &fidParm, &statParm,
+			RXAFS_BulkStatus(rxconn, &fidParm, &statParm,
 					 &cbParm, &volSync);
-		} else
-		    inlinebulk = 1;
+		    RX_AFS_GLOCK();
+		} else if (!code) {
+		    /* The InlineBulkStatus call itself succeeded, but we
+		     * may have failed to stat the first entry. Use the error
+		     * from the first entry for processing. */
+		    code = (&statsp[0])->errorCode;
+		}
 	    } else {
-		inlinebulk = 0;
+		RX_AFS_GUNLOCK();
 		code =
-		    RXAFS_BulkStatus(tcp->id, &fidParm, &statParm, &cbParm,
+		    RXAFS_BulkStatus(rxconn, &fidParm, &statParm, &cbParm,
 				     &volSync);
+		RX_AFS_GLOCK();
 	    }
-	    RX_AFS_GLOCK();
 	    XSTATS_END_TIME;
 	} else
 	    code = -1;
     } while (afs_Analyze
-	     (tcp, code, &adp->f.fid, areqp, AFS_STATS_FS_RPCIDX_BULKSTATUS,
+	     (tcp, rxconn, code, &adp->f.fid, areqp, AFS_STATS_FS_RPCIDX_BULKSTATUS,
 	      SHARED_LOCK, NULL));
 
     /* now, if we didnt get the info, bail out. */
@@ -958,6 +1030,9 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
   reskip:
     nskip = afs_cacheStats / 2;	/* preserved fraction of the cache */
     ObtainReadLock(&afs_xvcache);
+#ifdef AFS_DARWIN80_ENV
+ reskip2:
+#endif
     if (QEmpty(&VLRU)) {
 	/* actually a serious error, probably should panic. Probably will 
 	 * panic soon, oh well. */
@@ -969,9 +1044,13 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	refpanic("Bulkstat VLRU inconsistent");
     }
     for (tq = VLRU.next; tq != &VLRU; tq = QNext(tq)) {
-	if (--nskip <= 0)
-	    break;
-	else if (QNext(QPrev(tq)) != tq) {
+	if (--nskip <= 0) {
+#ifdef AFS_DARWIN80_ENV
+	    if ((!(QTOV(tq)->f.states & CDeadVnode)&&!(QTOV(tq)->f.states & CVInit)))
+#endif
+		break;
+	}
+	if (QNext(QPrev(tq)) != tq) {
 	    BStvc = QTOV(tq);
 	    refpanic("BulkStat VLRU inconsistent");
 	}
@@ -987,7 +1066,28 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
      * doesn't hurt nearly as much.
      */
     retry = 0;
+#ifdef AFS_DARWIN80_ENV
+    if (((lruvcp->f.states & CDeadVnode)||(lruvcp->f.states & CVInit))) {
+	if (npasses == 0) {
+	    nskip = 1;
+	    npasses++;
+	    goto reskip2;
+	} else
+	    panic("Can't find non-dead vnode in VLRU\n");
+    }
+    lruvp = AFSTOV(lruvcp);
+    if (vnode_get(lruvp))       /* this bumps ref count */
+	retry = 1;
+    else if (vnode_ref(lruvp)) {
+	AFS_GUNLOCK();
+	/* AFSTOV(lruvcp) may be NULL */
+	vnode_put(lruvp);
+	AFS_GLOCK();
+	retry = 1;
+    }
+#else
     osi_vnhold(lruvcp, &retry);
+#endif
     ReleaseReadLock(&afs_xvcache);	/* could be read lock */
     if (retry)
 	goto reskip;
@@ -1011,7 +1111,7 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	do {
 	    retry = 0;
 	    ObtainReadLock(&afs_xvcache);
-	    tvcp = afs_FindVCache(&afid, &retry, 0 /* !stats&!lru */ );
+	    tvcp = afs_FindVCache(&afid, &retry, 0/* !stats&!lru */);
 	    ReleaseReadLock(&afs_xvcache);
 	} while (tvcp && retry);
 
@@ -1027,9 +1127,12 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	 * matches the value we placed there when we set the CBulkFetching
 	 * flag, then someone else has done something with this node,
 	 * and we may not have the latest status information for this
-	 * file.  Leave the entry alone.
+	 * file.  Leave the entry alone. There's also a file type
+	 * change here, for OSX bulkstat support.
 	 */
-	if (!(tvcp->f.states & CBulkFetching) || (tvcp->f.m.Length != statSeqNo)) {
+	if (!(tvcp->f.states & CBulkFetching)
+	    || (tvcp->f.m.Length != statSeqNo)
+	    || (ftype[(&statsp[i])->FileType] != vType(tvcp))) {
 	    flagIndex++;
 	    ReleaseWriteLock(&tvcp->lock);
 	    afs_PutVCache(tvcp);
@@ -1044,6 +1147,11 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	    *tvcp->mvid = dotdot;
 	}
 
+#ifdef AFS_DARWIN80_ENV
+	if (((lruvcp->f.states & CDeadVnode)||(lruvcp->f.states & CVInit)))
+	    panic("vlru control point went dead\n");
+#endif
+
 	ObtainWriteLock(&afs_xvcache, 132);
 	if ((VLRU.next->prev != &VLRU) || (VLRU.prev->next != &VLRU)) {
 	    refpanic("Bulkstat VLRU inconsistent2");
@@ -1137,6 +1245,24 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	    if ((tvcp->f.states & CForeign) || (vType(tvcp) == VDIR))
 		osi_dnlc_purgedp(tvcp);	/* if it (could be) a directory */
 	}
+#ifdef AFS_DARWIN80_ENV
+	/* reclaim->FlushVCache will need xcbhash */
+	if (((tvcp->f.states & CDeadVnode)||(tvcp->f.states & CVInit))) {
+	    ReleaseWriteLock(&afs_xcbhash);
+	    /* passing in a parent hangs getting the vnode lock */
+	    code = afs_darwin_finalizevnode(tvcp, NULL, NULL, 0, 1);
+	    if (code) {
+		/* It's gonna get recycled - shouldn't happen */
+		tvcp->callback = 0;
+		tvcp->f.states &= ~(CStatd | CUnique);
+		afs_DequeueCallback(tvcp);
+		if ((tvcp->f.states & CForeign) || (vType(tvcp) == VDIR))
+		    osi_dnlc_purgedp(tvcp); /* if it (could be) a directory */
+	    } else
+		/* re-acquire the usecount that finalizevnode disposed of */
+		vnode_ref(AFSTOV(tvcp));
+	} else
+#endif
 	ReleaseWriteLock(&afs_xcbhash);
 
 	ReleaseWriteLock(&tvcp->lock);
@@ -1145,7 +1271,16 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
     }				/* for all files we got back */
 
     /* finally return the pointer into the LRU queue */
+#ifdef AFS_DARWIN80_ENV
+    if (((lruvcp->f.states & CDeadVnode)||(lruvcp->f.states & CVInit)))
+	panic("vlru control point went dead before put\n");
+    AFS_GUNLOCK();
+    vnode_put(lruvp);
+    vnode_rele(lruvp);
+    AFS_GLOCK();
+#else
     afs_PutVCache(lruvcp);
+#endif
 
   done:
     /* Be sure to turn off the CBulkFetching flags */
@@ -1157,30 +1292,20 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 	do {
 	    retry = 0;
 	    ObtainReadLock(&afs_xvcache);
-	    tvcp = afs_FindVCache(&afid, &retry, 0 /* !stats&!lru */ );
+	    tvcp = afs_FindVCache(&afid, &retry, 0 /* !stats&!lru */);
 	    ReleaseReadLock(&afs_xvcache);
 	} while (tvcp && retry);
-	if (tvcp != NULL && (tvcp->f.states & CBulkFetching)
-	    && (tvcp->f.m.Length == statSeqNo)) {
-	    tvcp->f.states &= ~CBulkFetching;
-	}
 	if (tvcp != NULL) {
+	    if ((tvcp->f.states & CBulkFetching)
+		&& (tvcp->f.m.Length == statSeqNo)) {
+		tvcp->f.states &= ~CBulkFetching;
+	    }
 	    afs_PutVCache(tvcp);
 	}
     }
     if (volp)
 	afs_PutVolume(volp, READ_LOCK);
 
-    /* If we did the InlineBulk RPC pull out the return code */
-    if (inlinebulk && code == 0) {
-	if ((&statsp[0])->errorCode) {
-	    afs_Analyze(tcp, (&statsp[0])->errorCode, &adp->f.fid, areqp,
-			AFS_STATS_FS_RPCIDX_BULKSTATUS, SHARED_LOCK, NULL);
-	    code = (&statsp[0])->errorCode;
-	}
-    } else {
-	code = 0;
-    }
   done2:
     osi_FreeLargeSpace((char *)fidsp);
     osi_Free((char *)statsp, AFSCBMAX * sizeof(AFSFetchStatus));
@@ -1190,27 +1315,42 @@ afs_DoBulkStat(struct vcache *adp, long dirCookie, struct vrequest *areqp)
 
 /* was: (AFS_DEC_ENV) || defined(AFS_OSF30_ENV) || defined(AFS_NCR_ENV) */
 #ifdef AFS_DARWIN80_ENV
-#define AFSDOBULK 0
+int AFSDOBULK = 1;
 #else
 static int AFSDOBULK = 1;
 #endif
 
+static_inline int
+osi_lookup_isdot(const char *aname)
+{
+#ifdef AFS_SUN5_ENV
+    if (!aname[0]) {
+	/* in Solaris, we can get passed "" as a path component if we are the
+	 * root directory, e.g. after a call to chroot. It is equivalent to
+	 * looking up "." */
+	return 1;
+    }
+#endif /* AFS_SUN5_ENV */
+    if (aname[0] == '.' && !aname[1]) {
+	return 1;
+    }
+    return 0;
+}
+
 int
-#ifdef AFS_OSF_ENV
-afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred, int opflag, int wantparent)
-#elif defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
-afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct pathname *pnp, int flags, struct vnode *rdir, AFS_UCRED *acred)
+#if defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, struct pathname *pnp, int flags, struct vnode *rdir, afs_ucred_t *acred)
 #elif defined(UKERNEL)
-afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred, int flags)
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, afs_ucred_t *acred, int flags)
 #else
-afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred)
+afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, afs_ucred_t *acred)
 #endif
 {
     struct vrequest treq;
     char *tname = NULL;
-    register struct vcache *tvc = 0;
-    register afs_int32 code;
-    register afs_int32 bulkcode = 0;
+    struct vcache *tvc = 0;
+    afs_int32 code;
+    afs_int32 bulkcode = 0;
     int pass = 0, hit = 0;
     int force_eval = afs_fakestat_enable ? 0 : 1;
     long dirCookie;
@@ -1231,10 +1371,6 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
     if ((code = afs_InitReq(&treq, acred)))
 	goto done;
 
-#ifdef	AFS_OSF_ENV
-    ndp->ni_dvp = AFSTOV(adp);
-#endif /* AFS_OSF_ENV */
-
     if (afs_fakestat_enable && adp->mvstat == 1) {
        if (strcmp(aname, ".directory") == 0)
            tryEvalOnly = 1;
@@ -1250,6 +1386,10 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 	if (strcmp(aname, "Contents") == 0)
 	    tryEvalOnly = 1;
     }
+    if (afs_fakestat_enable && adp->mvstat == 2) {
+	if (strncmp(aname, "._", 2) == 0)
+	    tryEvalOnly = 1;
+    }
 #endif
 
     if (tryEvalOnly)
@@ -1284,18 +1424,6 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
     if (adp->mvstat == 2 && aname[0] == '.' && aname[1] == '.' && !aname[2]) {
 	/* looking up ".." in root via special hacks */
 	if (adp->mvid == (struct VenusFid *)0 || adp->mvid->Fid.Volume == 0) {
-#ifdef	AFS_OSF_ENV
-	    if (adp == afs_globalVp) {
-		struct vnode *rvp = AFSTOV(adp);
-/*
-		ndp->ni_vp = rvp->v_vfsp->vfs_vnodecovered;
-		ndp->ni_dvp = ndp->ni_vp;
-		VN_HOLD(*avcp);
-*/
-		code = ENODEV;
-		goto done;
-	    }
-#endif
 	    code = ENODEV;
 	    goto done;
 	}
@@ -1336,7 +1464,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
      * I'm not fiddling with the LRUQ here, either, perhaps I should, or else 
      * invent a lightweight version of GetVCache.
      */
-    if (aname[0] == '.' && !aname[1]) {	/* special case */
+    if (osi_lookup_isdot(aname)) {	/* special case */
 	ObtainReadLock(&afs_xvcache);
 	osi_vnhold(adp, 0);
 	ReleaseReadLock(&afs_xvcache);
@@ -1379,15 +1507,26 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
      */
     if (afs_IsDynrootMount(adp)) {
 	struct VenusFid tfid;
-	afs_uint32 cellidx, volid, vnoid;
+	afs_uint32 cellidx, volid, vnoid, uniq;
 
-	code = EvalMountData('%', aname, 0, 0, NULL, &treq, &cellidx, &volid, &vnoid);
+	code = EvalMountData('%', aname, 0, 0, NULL, &treq, &cellidx, &volid, &vnoid, &uniq);
 	if (code)
 	    goto done;
-	afs_GetDynrootMountFid(&tfid);
-	tfid.Fid.Vnode = VNUM_FROM_TYPEID(VN_TYPE_MOUNT, cellidx << 2);
-	tfid.Fid.Unique = volid;
+	/* If a vnode was returned, it's not a real mount point */
+	if (vnoid > 1) {
+	    struct cell *tcell = afs_GetCellByIndex(cellidx, READ_LOCK);
+	    tfid.Cell = tcell->cellNum;
+	    afs_PutCell(tcell, READ_LOCK);
+	    tfid.Fid.Vnode = vnoid;
+	    tfid.Fid.Volume = volid;
+	    tfid.Fid.Unique = uniq;
+	} else {
+	    afs_GetDynrootMountFid(&tfid);
+	    tfid.Fid.Vnode = VNUM_FROM_TYPEID(VN_TYPE_MOUNT, cellidx << 2);
+	    tfid.Fid.Unique = volid;
+	}
 	*avcp = tvc = afs_GetVCache(&tfid, &treq, NULL, NULL);
+	code = (tvc ? 0 : ENOENT);
 	hit = 1;
 	goto done;
     }
@@ -1424,9 +1563,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
     if (tvc) {
 	if (no_read_access && vType(tvc) != VDIR && vType(tvc) != VLNK) {
 	    /* need read access on dir to stat non-directory / non-link */
-#ifndef AFS_FBSD80_ENV
 	    afs_PutVCache(tvc);
-#endif
 	    *avcp = NULL;
 	    code = EACCES;
 	    goto done;
@@ -1448,7 +1585,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
     }
 
     {				/* sub-block just to reduce stack usage */
-	register struct dcache *tdc;
+	struct dcache *tdc;
 	afs_size_t dirOffset, dirLen;
 	struct VenusFid tfid;
 
@@ -1530,9 +1667,22 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 	ReleaseReadLock(&tdc->lock);
 	if (!afs_InReadDir(adp))
 	    afs_PutDCache(tdc);
-
-	if (code == ENOENT && afs_IsDynroot(adp) && dynrootRetry) {
+	if (code == ENOENT && afs_IsDynroot(adp) && dynrootRetry && !tryEvalOnly) {
+	    struct cell *tc;
+	    char *cn = (tname[0] == '.') ? tname + 1 : tname;
 	    ReleaseReadLock(&adp->lock);
+	    /* confirm it's not just hushed */
+	    tc = afs_GetCellByName(cn, WRITE_LOCK);
+	    if (tc) {
+		if (tc->states & CHush) {
+		    tc->states &= ~CHush;
+		    ReleaseWriteLock(&tc->lock);
+		    afs_DynrootInvalidate();
+		    goto redo;
+		}
+		ReleaseWriteLock(&tc->lock);
+	    }
+	    /* Allow a second dynroot retry if the cell was hushed before */
 	    dynrootRetry = 0;
 	    if (tname[0] == '.')
 		afs_LookupAFSDB(tname + 1);
@@ -1554,7 +1704,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 
 	if (code) {
 	    if (code != ENOENT) {
-		printf("LOOKUP dirLookupOff -> %d\n", code);
+		/*printf("LOOKUP dirLookupOff -> %d\n", code);*/
 	    }
 	    goto done;
 	}
@@ -1583,9 +1733,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 
 	    /* if the vcache isn't usable, release it */
 	    if (tvc && !(tvc->f.states & CStatd)) {
-#ifndef  AFS_FBSD80_ENV
-	      afs_PutVCache(tvc);
-#endif
+		afs_PutVCache(tvc);
 		tvc = NULL;
 	    }
 	} else {
@@ -1635,10 +1783,10 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 	if (tvc->mvstat == 1 && (tvc->f.states & CMValid) && tvc->mvid != NULL)
 	  force_eval = 1; /* This is now almost for free, get it correct */
 
-#if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
+#if defined(UKERNEL)
 	if (!(flags & AFS_LOOKUP_NOEVAL))
 	    /* don't eval mount points */
-#endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
+#endif /* UKERNEL */
 	    if (tvc->mvstat == 1 && force_eval) {
 		/* a mt point, possibly unevaluated */
 		struct volume *tvolp;
@@ -1648,9 +1796,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 		ReleaseWriteLock(&tvc->lock);
 
 		if (code) {
-#ifndef AFS_FBSD80_ENV
 		    afs_PutVCache(tvc);
-#endif
 		    if (tvolp)
 			afs_PutVolume(tvolp, WRITE_LOCK);
 		    goto done;
@@ -1672,9 +1818,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 		    } else {
 			tvc = afs_GetVCache(tvc->mvid, &treq, NULL, NULL);
 		    }
-#ifndef AFS_FBSD80_ENV
 		    afs_PutVCache(uvc);	/* we're done with it */
-#endif
 
 		    if (!tvc) {
 			code = ENOENT;
@@ -1699,9 +1843,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 			afs_PutVolume(tvolp, WRITE_LOCK);
 		    }
 		} else {
-#ifndef AFS_FBSD80_ENV
 		    afs_PutVCache(tvc);
-#endif
 		    code = ENOENT;
 		    if (tvolp)
 			afs_PutVolume(tvolp, WRITE_LOCK);
@@ -1739,7 +1881,6 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 	    }
 	    code = ENOENT;
 	} else {
-	    printf("Network down in afs_lookup\n");
 	    code = ENETDOWN;
 	}
     }
@@ -1749,28 +1890,17 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
     if (tname != aname && tname)
 	osi_FreeLargeSpace(tname);
     if (code == 0) {
-#ifdef	AFS_OSF_ENV
-	/* Handle RENAME; only need to check rename "."  */
-	if (opflag == RENAME && wantparent && *ndp->ni_next == 0) {
-	    if (!FidCmp(&(tvc->f.fid), &(adp->f.fid))) {
-		afs_PutVCache(*avcp);
-		*avcp = NULL;
-		afs_PutFakeStat(&fakestate);
-		return afs_CheckCode(EISDIR, &treq, 18);
-	    }
-	}
-#endif /* AFS_OSF_ENV */
 
 	if (afs_mariner)
 	    afs_AddMarinerName(aname, tvc);
 
-#if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
+#if defined(UKERNEL)
 	if (!(flags & AFS_LOOKUP_NOEVAL)) {
 	    /* Here we don't enter the name into the DNLC because we want the
 	     * evaluated mount dir to be there (the vcache for the mounted
 	     * volume) rather than the vc of the mount point itself.  We can
 	     * still find the mount point's vc in the vcache by its fid. */
-#endif /* UKERNEL && AFS_WEB_ENHANCEMENTS */
+#endif /* UKERNEL */
 	    if (!hit && force_eval) {
 		osi_dnlc_enter(adp, aname, tvc, &versionNo);
 	    } else {
@@ -1783,7 +1913,7 @@ afs_lookup(OSI_VC_DECL(adp), char *aname, struct vcache **avcp, AFS_UCRED *acred
 		return 0;	/* can't have been any errors if hit and !code */
 #endif
 	    }
-#if defined(UKERNEL) && defined(AFS_WEB_ENHANCEMENTS)
+#if defined(UKERNEL)
 	}
 #endif
     }
