@@ -37,7 +37,6 @@ extern afs_rwlock_t afs_xcbhash;
  * is just a performance hit.
  */
 
-#ifdef AFS_DISCON_ENV
 static int
 afs_DisconCreateSymlink(struct vcache *avc, char *aname, 
 		        struct vrequest *areq) {
@@ -47,7 +46,7 @@ afs_DisconCreateSymlink(struct vcache *avc, char *aname,
 
     tdc = afs_GetDCache(avc, 0, areq, &offset, &len, 0);
     if (!tdc) {
-	printf("afs_DisconCreateSymlink: can't get new dcache for symlink.\n");
+	/* printf("afs_DisconCreateSymlink: can't get new dcache for symlink.\n"); */
 	return ENOENT;
     }
 
@@ -63,12 +62,11 @@ afs_DisconCreateSymlink(struct vcache *avc, char *aname,
     ReleaseWriteLock(&tdc->lock);
     return 0;
 }
-#endif
 
 /* don't set CDirty in here because RPC is called synchronously */
 int 
 afs_symlink(OSI_VC_DECL(adp), char *aname, struct vattr *attrs, 
-	    char *atargetName, struct AFS_UCRED *acred)
+	    char *atargetName, afs_ucred_t *acred)
 {
     afs_uint32 now = 0;
     struct vrequest treq;
@@ -86,6 +84,7 @@ afs_symlink(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     struct AFSVolSync tsync;
     struct volume *volp = 0;
     struct afs_fakestat_state fakestate;
+    struct rx_connection *rxconn;
     XSTATS_DECLS;
     OSI_VC_CONVERT(adp);
 
@@ -153,19 +152,20 @@ afs_symlink(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     ObtainWriteLock(&adp->lock, 156);
     if (tdc)
 	ObtainWriteLock(&tdc->lock, 636);
-    ObtainSharedLock(&afs_xvcache, 17);	/* prevent others from creating this entry */
-    /* XXX Pay attention to afs_xvcache around the whole thing!! XXX */
+    /* No further locks: if the SymLink succeeds, it does not matter what happens
+     * to our local copy of the directory. If somebody tampers with it in the meantime,
+     * the copy will be invalidated */
     if (!AFS_IS_DISCON_RW) {
 	do {
-	    tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK);
+	    tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK, &rxconn);
 	    if (tc) {
-		hostp = tc->srvr->server;
+		hostp = tc->parent->srvr->server;
 		XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_SYMLINK);
 		if (adp->f.states & CForeign) {
 		    now = osi_Time();
 		    RX_AFS_GUNLOCK();
 		    code = 
-			RXAFS_DFSSymlink(tc->id, 
+			RXAFS_DFSSymlink(rxconn,
 					 (struct AFSFid *)&adp->f.fid.Fid,
 					 aname, atargetName, &InStatus,
 					 (struct AFSFid *)&newFid.Fid,
@@ -175,7 +175,7 @@ afs_symlink(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 		} else {
 		    RX_AFS_GUNLOCK();
 		    code =
-			RXAFS_Symlink(tc->id, (struct AFSFid *)&adp->f.fid.Fid,
+			RXAFS_Symlink(rxconn, (struct AFSFid *)&adp->f.fid.Fid,
 				      aname, atargetName, &InStatus,
 				      (struct AFSFid *)&newFid.Fid, 
 				      &OutFidStatus, &OutDirStatus, &tsync);
@@ -185,17 +185,15 @@ afs_symlink(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	    } else
 		code = -1;
 	} while (afs_Analyze
-		    (tc, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_SYMLINK,
+		    (tc, rxconn, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_SYMLINK,
 		     SHARED_LOCK, NULL));
     } else {
-#ifdef AFS_DISCON_ENV
 	newFid.Cell = adp->f.fid.Cell;
 	newFid.Fid.Volume = adp->f.fid.Fid.Volume;
 	afs_GenFakeFid(&newFid, VREG, 0);
-#endif
     }
 
-    UpgradeSToWLock(&afs_xvcache, 40);
+    ObtainWriteLock(&afs_xvcache, 40);
     if (code) {
 	if (code < 0) {
 	    ObtainWriteLock(&afs_xcbhash, 499);
@@ -216,6 +214,8 @@ afs_symlink(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     if (AFS_IS_DISCON_RW || afs_LocalHero(adp, tdc, &OutDirStatus, 1)) {
 	/* we can do it locally */
 	ObtainWriteLock(&afs_xdcache, 293);
+	/* If the following fails because the name has been created in the meantime, the
+	 * directory is out-of-date - the file server knows best! */
 	code = afs_dir_Create(tdc, aname, &newFid.Fid);
 	ReleaseWriteLock(&afs_xdcache);
 	if (code && !AFS_IS_DISCON_RW) {
@@ -260,7 +260,6 @@ afs_symlink(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
     ReleaseWriteLock(&afs_xcbhash);
 
     if (AFS_IS_DISCON_RW) {
-#ifdef AFS_DISCON_ENV
 	attrs->va_mode = InStatus.UnixModeBits;
 	afs_GenDisconStatus(adp, tvc, &newFid, attrs, &treq, VLNK);
 	code = afs_DisconCreateSymlink(tvc, atargetName, &treq);
@@ -273,13 +272,13 @@ afs_symlink(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 	    goto done;
 	}
 	afs_DisconAddDirty(tvc, VDisconCreate, 0);
-#endif
     } else {
 	afs_ProcessFS(tvc, &OutFidStatus, &treq);
     }
 
     if (!tvc->linkData) {
-	tvc->linkData = (char *)afs_osi_Alloc(alen);
+	tvc->linkData = afs_osi_Alloc(alen);
+	osi_Assert(tvc->linkData != NULL);
 	strncpy(tvc->linkData, atargetName, alen - 1);
 	tvc->linkData[alen - 1] = 0;
     }
@@ -298,13 +297,13 @@ afs_symlink(OSI_VC_DECL(adp), char *aname, struct vattr *attrs,
 }
 
 int
-afs_MemHandleLink(register struct vcache *avc, struct vrequest *areq)
+afs_MemHandleLink(struct vcache *avc, struct vrequest *areq)
 {
-    register struct dcache *tdc;
-    register char *tp, *rbuf;
+    struct dcache *tdc;
+    char *tp, *rbuf;
     afs_size_t offset, len;
     afs_int32 tlen, alen;
-    register afs_int32 code;
+    afs_int32 code;
 
     AFS_STATCNT(afs_MemHandleLink);
     /* two different formats, one for links protected 644, have a "." at
@@ -336,6 +335,7 @@ afs_MemHandleLink(register struct vcache *avc, struct vrequest *areq)
 	rbuf[alen - 1] = 0;
 	alen = strlen(rbuf) + 1;
 	tp = afs_osi_Alloc(alen);	/* make room for terminating null */
+	osi_Assert(tp != NULL);
 	memcpy(tp, rbuf, alen);
 	osi_FreeLargeSpace(rbuf);
 	if (code != len) {
@@ -348,14 +348,14 @@ afs_MemHandleLink(register struct vcache *avc, struct vrequest *areq)
 }
 
 int
-afs_UFSHandleLink(register struct vcache *avc, struct vrequest *areq)
+afs_UFSHandleLink(struct vcache *avc, struct vrequest *areq)
 {
-    register struct dcache *tdc;
-    register char *tp, *rbuf;
+    struct dcache *tdc;
+    char *tp, *rbuf;
     void *tfile;
     afs_size_t offset, len;
     afs_int32 tlen, alen;
-    register afs_int32 code;
+    afs_int32 code;
 
     /* two different formats, one for links protected 644, have a "." at the
      * end of the file name, which we turn into a null.  Others, protected
@@ -392,6 +392,7 @@ afs_UFSHandleLink(register struct vcache *avc, struct vrequest *areq)
 	rbuf[alen - 1] = '\0';
 	alen = strlen(rbuf) + 1;
 	tp = afs_osi_Alloc(alen);	/* make room for terminating null */
+	osi_Assert(tp != NULL);
 	memcpy(tp, rbuf, alen);
 	osi_FreeLargeSpace(rbuf);
 	if (code != tlen) {
@@ -404,11 +405,11 @@ afs_UFSHandleLink(register struct vcache *avc, struct vrequest *areq)
 }
 
 int
-afs_readlink(OSI_VC_DECL(avc), struct uio *auio, struct AFS_UCRED *acred)
+afs_readlink(OSI_VC_DECL(avc), struct uio *auio, afs_ucred_t *acred)
 {
-    register afs_int32 code;
+    afs_int32 code;
     struct vrequest treq;
-    register char *tp;
+    char *tp;
     struct afs_fakestat_state fakestat;
     OSI_VC_CONVERT(avc);
 
