@@ -19,7 +19,6 @@
 #undef kmem_free
 #undef mem_alloc
 #undef mem_free
-#undef register
 #endif /* AFS_ALPHA_ENV */
 #include "afsincludes.h"	/* Afs-based standard headers */
 #include "afs/afs_stats.h"	/* statistics */
@@ -27,11 +26,10 @@
 
 extern int cacheDiskType;
 
-
 #ifndef AFS_NOSTATS
-void
-FillStoreStats(int code, int idx, osi_timeval_t *xferStartTime,
-		  afs_size_t bytesToXfer, afs_size_t bytesXferred)
+static void
+FillStoreStats(int code, int idx, osi_timeval_t xferStartTime,
+	       afs_size_t bytesToXfer, afs_size_t bytesXferred)
 {
     struct afs_stats_xferData *xferP;
     osi_timeval_t xferStopTime;
@@ -65,7 +63,7 @@ FillStoreStats(int code, int idx, osi_timeval_t *xferStartTime,
 	else
 	    (xferP->count[8])++;
 
-	afs_stats_GetDiff(elapsedTime, (*xferStartTime), xferStopTime);
+	afs_stats_GetDiff(elapsedTime, xferStartTime, xferStopTime);
 	afs_stats_AddTo((xferP->sumTime), elapsedTime);
 	afs_stats_SquareAddTo((xferP->sqrTime), elapsedTime);
 	if (afs_stats_TimeLessThan(elapsedTime, (xferP->minTime))) {
@@ -80,14 +78,7 @@ FillStoreStats(int code, int idx, osi_timeval_t *xferStartTime,
 
 /* rock and operations for RX_FILESERVER */
 
-struct rxfs_storeVariables {
-    struct rx_call *call;
-    char *tbuffer;
-    struct iovec *tiov;
-    afs_int32 tnio;
-    afs_int32 hasNo64bit;
-    struct AFSStoreStatus InStatus;
-};
+
 
 afs_int32
 rxfs_storeUfsPrepare(void *r, afs_uint32 size, afs_uint32 *tlen)
@@ -102,9 +93,8 @@ rxfs_storeMemPrepare(void *r, afs_uint32 size, afs_uint32 *tlen)
     afs_int32 code;
     struct rxfs_storeVariables *v = (struct rxfs_storeVariables *) r;
 
-    *tlen = (size > AFS_LRALLOCSIZ ?  AFS_LRALLOCSIZ : size);
     RX_AFS_GUNLOCK();
-    code = rx_WritevAlloc(v->call, v->tiov, &v->tnio, RX_MAXIOVECS, *tlen);
+    code = rx_WritevAlloc(v->call, v->tiov, &v->tnio, RX_MAXIOVECS, size);
     RX_AFS_GLOCK();
     if (code <= 0) {
 	code = rx_Error(v->call);
@@ -213,7 +203,7 @@ rxfs_storePadd(void *rock, afs_uint32 size)
 	    return -33;	/* XXX */
 	size -= tlen;
     }
-    return code;
+    return 0;
 }
 
 afs_int32
@@ -271,30 +261,108 @@ rxfs_storeDestroy(void **r, afs_int32 error)
     return code;
 }
 
+afs_int32
+afs_GenericStoreProc(struct storeOps *ops, void *rock,
+		     struct dcache *tdc, int *shouldwake,
+		     afs_size_t *bytesXferred)
+{
+    struct rxfs_storeVariables *svar = rock;
+    afs_uint32 tlen, bytesread, byteswritten;
+    afs_int32 code = 0;
+    int offset = 0;
+    afs_size_t size;
+    struct osi_file *tfile;
+
+    size = tdc->f.chunkBytes;
+
+    tfile = afs_CFileOpen(&tdc->f.inode);
+
+    while ( size > 0 ) {
+	code = (*ops->prepare)(rock, size, &tlen);
+	if ( code )
+	    break;
+
+	code = (*ops->read)(rock, tfile, offset, tlen, &bytesread);
+	if (code)
+	    break;
+
+	tlen = bytesread;
+	code = (*ops->write)(rock, tlen, &byteswritten);
+	if (code)
+	    break;
+#ifndef AFS_NOSTATS
+	*bytesXferred += byteswritten;
+#endif /* AFS_NOSTATS */
+
+	offset += tlen;
+	size -= tlen;
+	/*
+	 * if file has been locked on server, can allow
+	 * store to continue
+	 */
+	if (shouldwake && *shouldwake && ((*ops->status)(rock) == 0)) {
+	    *shouldwake = 0;	/* only do this once */
+	    afs_wakeup(svar->vcache);
+	}
+    }
+    afs_CFileClose(tfile);
+
+    return code;
+}
+
 static
 struct storeOps rxfs_storeUfsOps = {
+#ifndef HAVE_STRUCT_LABEL_SUPPORT
     rxfs_storeUfsPrepare,
     rxfs_storeUfsRead,
     rxfs_storeUfsWrite,
     rxfs_storeStatus,
     rxfs_storePadd,
     rxfs_storeClose,
-    rxfs_storeDestroy
+    rxfs_storeDestroy,
+    afs_GenericStoreProc
+#else
+    .prepare = 	rxfs_storeUfsPrepare,
+    .read =	rxfs_storeUfsRead,
+    .write =	rxfs_storeUfsWrite,
+    .status =	rxfs_storeStatus,
+    .padd =	rxfs_storePadd,
+    .close =	rxfs_storeClose,
+    .destroy =	rxfs_storeDestroy,
+#ifdef AFS_LINUX26_ENV
+    .storeproc = afs_linux_storeproc
+#else
+    .storeproc = afs_GenericStoreProc
+#endif
+#endif
 };
 
 static
 struct storeOps rxfs_storeMemOps = {
+#ifndef HAVE_STRUCT_LABEL_SUPPORT
     rxfs_storeMemPrepare,
     rxfs_storeMemRead,
     rxfs_storeMemWrite,
     rxfs_storeStatus,
     rxfs_storePadd,
     rxfs_storeClose,
-    rxfs_storeDestroy
+    rxfs_storeDestroy,
+    afs_GenericStoreProc
+#else
+    .prepare =	rxfs_storeMemPrepare,
+    .read = 	rxfs_storeMemRead,
+    .write = 	rxfs_storeMemWrite,
+    .status =	rxfs_storeStatus,
+    .padd =	rxfs_storePadd,
+    .close = 	rxfs_storeClose,
+    .destroy =	rxfs_storeDestroy,
+    .storeproc = afs_GenericStoreProc
+#endif
 };
 
 afs_int32
-rxfs_storeInit(struct vcache *avc, struct afs_conn *tc, afs_size_t base,
+rxfs_storeInit(struct vcache *avc, struct afs_conn *tc,
+                struct rx_connection *rxconn, afs_size_t base,
 		afs_size_t bytes, afs_size_t length,
 		int sync, struct storeOps **ops, void **rock)
 {
@@ -311,10 +379,11 @@ rxfs_storeInit(struct vcache *avc, struct afs_conn *tc, afs_size_t base,
 
     v->InStatus.ClientModTime = avc->f.m.Date;
     v->InStatus.Mask = AFS_SETMODTIME;
+    v->vcache = avc;
     if (sync & AFS_SYNC)
         v->InStatus.Mask |= AFS_FSYNC;
     RX_AFS_GUNLOCK();
-    v->call = rx_NewCall(tc->id);
+    v->call = rx_NewCall(rxconn);
     if (v->call) {
 #ifdef AFS_64BIT_CLIENT
 	if (!afs_serverHasNo64Bit(tc))
@@ -371,7 +440,6 @@ rxfs_storeInit(struct vcache *avc, struct afs_conn *tc, afs_size_t base,
     *rock = (void *)v;
     return 0;
 }
-
 unsigned int storeallmissing = 0;
 /*!
  *	Called for each chunk upon store.
@@ -389,36 +457,33 @@ unsigned int storeallmissing = 0;
  */
 afs_int32
 afs_CacheStoreDCaches(struct vcache *avc, struct dcache **dclist,
-			afs_size_t bytes,
-			afs_hyper_t *anewDV,
-			int *doProcessFS,
-			struct AFSFetchStatus *OutStatus,
-			afs_uint32 nchunks,
-			int nomore,
-			struct storeOps *ops, void *rock)
+		      afs_size_t bytes, afs_hyper_t *anewDV, int *doProcessFS,
+		      struct AFSFetchStatus *OutStatus, afs_uint32 nchunks,
+		      int nomore, struct storeOps *ops, void *rock)
 {
     int *shouldwake = NULL;
     unsigned int i;
     afs_int32 code = 0;
+    afs_size_t bytesXferred;
 
 #ifndef AFS_NOSTATS
     osi_timeval_t xferStartTime;	/*FS xfer start time */
     afs_size_t bytesToXfer = 10000;	/* # bytes to xfer */
-    afs_size_t bytesXferred = 10000;	/* # bytes actually xferred */
 #endif /* AFS_NOSTATS */
     XSTATS_DECLS;
+    osi_Assert(nchunks != 0);
 
     for (i = 0; i < nchunks && !code; i++) {
 	int stored = 0;
-	struct osi_file *tfile;
-	int offset = 0;
 	struct dcache *tdc = dclist[i];
-	afs_int32 size = tdc->f.chunkBytes;
+	afs_int32 size;
+
 	if (!tdc) {
 	    afs_warn("afs: missing dcache!\n");
 	    storeallmissing++;
 	    continue;	/* panic? */
 	}
+	size = tdc->f.chunkBytes;
 	afs_Trace4(afs_iclSetp, CM_TRACE_STOREALL2, ICL_TYPE_POINTER, avc,
 		    ICL_TYPE_INT32, tdc->f.chunk, ICL_TYPE_INT32, tdc->index,
 		    ICL_TYPE_INT32, afs_inode2trace(&tdc->f.inode));
@@ -431,7 +496,6 @@ afs_CacheStoreDCaches(struct vcache *avc, struct dcache **dclist,
 	    else if ((afs_uint32) avc->asynchrony >= (bytes - stored))
 		shouldwake = &nomore;
 	}
-	tfile = afs_CFileOpen(&tdc->f.inode);
 
 	afs_Trace4(afs_iclSetp, CM_TRACE_STOREPROC, ICL_TYPE_POINTER, avc,
 		    ICL_TYPE_FID, &(avc->f.fid), ICL_TYPE_OFFSET,
@@ -447,51 +511,23 @@ afs_CacheStoreDCaches(struct vcache *avc, struct dcache **dclist,
 	 * to ship here.
 	 */
 	bytesToXfer = size;
-	bytesXferred = 0;
 
 	osi_GetuTime(&xferStartTime);
 #endif /* AFS_NOSTATS */
+	bytesXferred = 0;
 
-	while ( size > 0 ) {
-	    afs_uint32 tlen;
-	    afs_uint32 bytesread, byteswritten;
-	    code = (*ops->prepare)(rock, size, &tlen);
-	    if ( code )
-		break;
+	code = (*ops->storeproc)(ops, rock, tdc, shouldwake,
+				     &bytesXferred);
 
-	    code = (*ops->read)(rock, tfile, offset, tlen, &bytesread);
-	    if (code)
-		break;
-
-	    tlen = bytesread;
-	    code = (*ops->write)(rock, tlen, &byteswritten);
-	    if (code)
-		break;
-#ifndef AFS_NOSTATS
-	    bytesXferred += byteswritten;
-#endif /* AFS_NOSTATS */
-
-	    offset += tlen;
-	    size -= tlen;
-	    /*
-	     * if file has been locked on server, can allow
-	     * store to continue
-	     */
-	    if (shouldwake && *shouldwake && ((*ops->status)(rock) == 0)) {
-		*shouldwake = 0;	/* only do this once */
-		afs_wakeup(avc);
-	    }
-	}
 	afs_Trace4(afs_iclSetp, CM_TRACE_STOREPROC, ICL_TYPE_POINTER, avc,
 		    ICL_TYPE_FID, &(avc->f.fid), ICL_TYPE_OFFSET,
 		    ICL_HANDLE_OFFSET(avc->f.m.Length), ICL_TYPE_INT32, size);
 
 #ifndef AFS_NOSTATS
 	FillStoreStats(code, AFS_STATS_FS_XFERIDX_STOREDATA,
-		    &xferStartTime, bytesToXfer, bytesXferred);
+		    xferStartTime, bytesToXfer, bytesXferred);
 #endif /* AFS_NOSTATS */
 
-	afs_CFileClose(tfile);
 	if ((tdc->f.chunkBytes < afs_OtherCSize)
 		&& (i < (nchunks - 1)) && code == 0) {
 	    code = (*ops->padd)(rock, afs_OtherCSize - tdc->f.chunkBytes);
@@ -535,10 +571,9 @@ afs_CacheStoreDCaches(struct vcache *avc, struct dcache **dclist,
  */
 int
 afs_CacheStoreVCache(struct dcache **dcList, struct vcache *avc,
-			struct vrequest *areq, int sync,
-			unsigned int minj, unsigned int high,
-			unsigned int moredata,
-			afs_hyper_t *anewDV, afs_size_t *amaxStoredLength)
+		     struct vrequest *areq, int sync, unsigned int minj,
+		     unsigned int high, unsigned int moredata,
+		     afs_hyper_t *anewDV, afs_size_t *amaxStoredLength)
 {
     afs_int32 code = 0;
     struct storeOps *ops;
@@ -551,6 +586,7 @@ afs_CacheStoreVCache(struct dcache **dcList, struct vcache *avc,
     int nomore;
     unsigned int first = 0;
     struct afs_conn *tc;
+    struct rx_connection *rxconn;
 
     for (bytes = 0, j = 0; !code && j <= high; j++) {
 	if (dcList[j]) {
@@ -593,31 +629,29 @@ afs_CacheStoreVCache(struct dcache **dcList, struct vcache *avc,
 		       ICL_HANDLE_OFFSET(base), ICL_TYPE_OFFSET,
 		       ICL_HANDLE_OFFSET(bytes), ICL_TYPE_OFFSET,
 		       ICL_HANDLE_OFFSET(length));
-	    tc = afs_Conn(&avc->f.fid, areq, 0);
 
 	    do {
+	        tc = afs_Conn(&avc->f.fid, areq, 0, &rxconn);
+
 #ifdef AFS_64BIT_CLIENT
 	      restart:
 #endif
-		code = rxfs_storeInit(avc, tc, base, bytes, length,
-					sync, &ops, &rock);
-		if ( code )
-		    goto nocall;
+		code = rxfs_storeInit(avc, tc, rxconn, base, bytes, length,
+				      sync, &ops, &rock);
+		if ( !code ) {
+		    code = afs_CacheStoreDCaches(avc, dclist, bytes, anewDV,
+			                         &doProcessFS, &OutStatus,
+						 nchunks, nomore, ops, rock);
+		}
 
-		code = afs_CacheStoreDCaches(avc, dclist, bytes, anewDV,
-			&doProcessFS, &OutStatus, nchunks, nomore, ops, rock);
-
-nocall:
 #ifdef AFS_64BIT_CLIENT
 		if (code == RXGEN_OPCODE && !afs_serverHasNo64Bit(tc)) {
 		    afs_serverSetNo64Bit(tc);
 		    goto restart;
 		}
-#else
-		continue;	/* dummy, label before block end won't work */
 #endif /* AFS_64BIT_CLIENT */
 	    } while (afs_Analyze
-		     (tc, code, &avc->f.fid, areq,
+		     (tc, rxconn, code, &avc->f.fid, areq,
 		      AFS_STATS_FS_RPCIDX_STOREDATA, SHARED_LOCK,
 		      NULL));
 
@@ -734,9 +768,8 @@ rxfs_fetchMemRead(void *r, afs_uint32 tlen, afs_uint32 *bytesread)
 
 
 afs_int32
-rxfs_fetchMemWrite(void *r, struct osi_file *fP,
-			afs_uint32 offset, afs_uint32 tlen,
-			afs_uint32 *byteswritten)
+rxfs_fetchMemWrite(void *r, struct osi_file *fP, afs_uint32 offset,
+		   afs_uint32 tlen, afs_uint32 *byteswritten)
 {
     afs_int32 code;
     struct rxfs_fetchVariables *v = (struct rxfs_fetchVariables *)r;
@@ -751,9 +784,8 @@ rxfs_fetchMemWrite(void *r, struct osi_file *fP,
 }
 
 afs_int32
-rxfs_fetchUfsWrite(void *r, struct osi_file *fP,
-			afs_uint32 offset, afs_uint32 tlen,
-			afs_uint32 *byteswritten)
+rxfs_fetchUfsWrite(void *r, struct osi_file *fP, afs_uint32 offset,
+		   afs_uint32 tlen, afs_uint32 *byteswritten)
 {
     afs_int32 code;
     struct rxfs_fetchVariables *v = (struct rxfs_fetchVariables *)r;
@@ -769,7 +801,7 @@ rxfs_fetchUfsWrite(void *r, struct osi_file *fP,
 
 afs_int32
 rxfs_fetchClose(void *r, struct vcache *avc, struct dcache * adc,
-					struct afs_FetchOutput *o)
+		struct afs_FetchOutput *o)
 {
     afs_int32 code, code1 = 0;
     struct rxfs_fetchVariables *v = (struct rxfs_fetchVariables *)r;
@@ -868,24 +900,26 @@ struct fetchOps rxfs_fetchMemOps = {
 };
 
 afs_int32
-rxfs_fetchInit(struct afs_conn *tc, struct vcache *avc, afs_offs_t base,
-		afs_uint32 size, afs_int32 *alength, struct dcache *adc,
-		struct osi_file *fP, struct fetchOps **ops, void **rock)
+rxfs_fetchInit(struct afs_conn *tc, struct rx_connection *rxconn,
+               struct vcache *avc, afs_offs_t base,
+	       afs_uint32 size, afs_int32 *alength, struct dcache *adc,
+	       struct osi_file *fP, struct fetchOps **ops, void **rock)
 {
     struct rxfs_fetchVariables *v;
-    int code = 0, code1;
+    int code = 0, code1 = 0;
 #ifdef AFS_64BIT_CLIENT
     afs_uint32 length_hi = 0;
 #endif
     afs_uint32 length, bytes;
 
-    v = (struct rxfs_fetchVariables *) osi_AllocSmallSpace(sizeof(struct rxfs_fetchVariables));
+    v = (struct rxfs_fetchVariables *)
+	    osi_AllocSmallSpace(sizeof(struct rxfs_fetchVariables));
     if (!v)
         osi_Panic("rxfs_fetchInit: osi_AllocSmallSpace returned NULL\n");
     memset(v, 0, sizeof(struct rxfs_fetchVariables));
 
     RX_AFS_GUNLOCK();
-    v->call = rx_NewCall(tc->id);
+    v->call = rx_NewCall(rxconn);
     RX_AFS_GLOCK();
     if (v->call) {
 #ifdef AFS_64BIT_CLIENT
@@ -893,8 +927,9 @@ rxfs_fetchInit(struct afs_conn *tc, struct vcache *avc, afs_offs_t base,
 	if (!afs_serverHasNo64Bit(tc)) {
 	    afs_uint64 llbytes = size;
 	    RX_AFS_GUNLOCK();
-	    code = StartRXAFS_FetchData64(v->call, (struct AFSFid *) &avc->f.fid.Fid,
-					       base, llbytes);
+	    code = StartRXAFS_FetchData64(v->call,
+					  (struct AFSFid *) &avc->f.fid.Fid,
+					  base, llbytes);
 	    if (code != 0) {
 		RX_AFS_GLOCK();
 		afs_Trace2(afs_iclSetp, CM_TRACE_FETCH64CODE,
@@ -921,7 +956,7 @@ rxfs_fetchInit(struct afs_conn *tc, struct vcache *avc, afs_offs_t base,
 		pos = base;
 		RX_AFS_GUNLOCK();
 		if (!v->call)
-		    v->call = rx_NewCall(tc->id);
+		    v->call = rx_NewCall(rxconn);
 		code =
 		    StartRXAFS_FetchData(
 		    		v->call, (struct AFSFid*)&avc->f.fid.Fid,
@@ -971,6 +1006,26 @@ rxfs_fetchInit(struct afs_conn *tc, struct vcache *avc, afs_offs_t base,
 #endif /* AFS_64BIT_CLIENT */
     } else
 	code = -1;
+
+    /* We need to cast here, in order to avoid issues if *alength is
+     * negative. Some, older, fileservers can return a negative length,
+     * which the rest of the code deals correctly with. */
+    if (code == 0 && *alength > (afs_int32) size) {
+	/* The fileserver told us it is going to send more data than we
+	 * requested. It shouldn't do that, and accepting that much data
+	 * can make us take up more cache space than we're supposed to,
+	 * so error. */
+	code = rx_Error(v->call);
+	RX_AFS_GUNLOCK();
+	code1 = rx_EndCall(v->call, code);
+	RX_AFS_GLOCK();
+	v->call = NULL;
+	code = EIO;
+    }
+
+    if (!code && code1)
+	code = code1;
+
     if (code) {
 	osi_FreeSmallSpace(v);
         return code;
@@ -1005,7 +1060,8 @@ rxfs_fetchInit(struct afs_conn *tc, struct vcache *avc, afs_offs_t base,
  * Routine called on fetch; also tells people waiting for data
  *	that more has arrived.
  *
- * \param tc Ptr to the Rx connection structure.
+ * \param tc Ptr to the AFS connection structure.
+ * \param rxconn Ptr to the Rx connection structure.
  * \param fP File descriptor for the cache file.
  * \param base Base offset to fetch.
  * \param adc Ptr to the dcache entry for the file, write-locked.
@@ -1016,9 +1072,10 @@ rxfs_fetchInit(struct afs_conn *tc, struct vcache *avc, afs_offs_t base,
  * \note Environment: Nothing interesting.
  */
 int
-afs_CacheFetchProc(struct afs_conn *tc, struct osi_file *fP, afs_size_t base,
-		    struct dcache *adc, struct vcache *avc, afs_int32 size,
-		    struct afs_FetchOutput *tsmall)
+afs_CacheFetchProc(struct afs_conn *tc, struct rx_connection *rxconn,
+                   struct osi_file *fP, afs_size_t base,
+		   struct dcache *adc, struct vcache *avc, afs_int32 size,
+		   struct afs_FetchOutput *tsmall)
 {
     afs_int32 code;
     afs_int32 length;
@@ -1045,7 +1102,7 @@ afs_CacheFetchProc(struct afs_conn *tc, struct osi_file *fP, afs_size_t base,
      * adc->lock(W)
      */
     code = rxfs_fetchInit(
-		tc, avc, base, size, &length, adc, fP, &ops, &rock);
+		tc, rxconn, avc, base, size, &length, adc, fP, &ops, &rock);
 
 #ifndef AFS_NOSTATS
     osi_GetuTime(&xferStartTime);
@@ -1105,7 +1162,7 @@ afs_CacheFetchProc(struct afs_conn *tc, struct osi_file *fP, afs_size_t base,
 	(*ops->destroy)(&rock, code);
 
 #ifndef AFS_NOSTATS
-    FillStoreStats(code, AFS_STATS_FS_XFERIDX_FETCHDATA, &xferStartTime,
+    FillStoreStats(code, AFS_STATS_FS_XFERIDX_FETCHDATA, xferStartTime,
 			bytesToXfer, bytesXferred);
 #endif
     XSTATS_END_TIME;
