@@ -13,12 +13,10 @@
 
 #include <afsconfig.h>
 #include <afs/param.h>
-#ifndef AFS_NT40_ENV
-#include <netdb.h>
-#endif
 
-RCSID
-    ("$Header: /cvs/openafs/src/libadmin/test/vos.c,v 1.9 2005/10/23 19:05:40 jaltman Exp $");
+#include <roken.h>
+
+#include <afs/afsutil.h>
 
 #include "vos.h"
 
@@ -69,7 +67,7 @@ GetVolumeIdFromString(const char *volume)
      * is a volume name
      */
     if (vos_VLDBGet
-	(cellHandle, 0, (const unsigned int *)0, volume, &entry, &st)) {
+	(cellHandle, 0, NULL, (char *)volume, &entry, &st)) {
 	return entry.volumeId[VOS_READ_WRITE_VOLUME];
     } else {
 	ERR_EXT("failed to convert specified volume to an id");
@@ -137,18 +135,20 @@ GetAddressFromString(const char *addr_str)
     return ntohl(addr);
 }
 
+#if 0
 static void
 PrintMessage(vos_messageType_t type, char *message)
 {
     printf("%s\n", message);
 }
+#endif
 
 int
-DoVosBackupVolumeCreate(struct cmd_syndesc *as, char *arock)
+DoVosBackupVolumeCreate(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { VOLUME } DoVosBackupVolumeCreate_parm_t;
     afs_status_t st = 0;
-    unsigned int volume_id;
+    unsigned int volume_id = 0;
 
     if (as->parms[VOLUME].items) {
 	const char *volume = as->parms[VOLUME].items->data;
@@ -162,7 +162,7 @@ DoVosBackupVolumeCreate(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosBackupVolumeCreateMultiple(struct cmd_syndesc *as, char *arock)
+DoVosBackupVolumeCreateMultiple(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, PREFIX,
 	EXCLUDE
@@ -214,12 +214,12 @@ Print_vos_partitionEntry_p(vos_partitionEntry_p part, const char *prefix)
 }
 
 int
-DoVosPartitionGet(struct cmd_syndesc *as, char *arock)
+DoVosPartitionGet(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION } DoVosPartitionGet_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
+    unsigned int partition_id = 0;
     vos_partitionEntry_t entry;
 
     if (as->parms[SERVER].items) {
@@ -245,7 +245,7 @@ DoVosPartitionGet(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosPartitionList(struct cmd_syndesc *as, char *arock)
+DoVosPartitionList(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER } DoVosPartitionGet_parm_t;
     afs_status_t st = 0;
@@ -281,7 +281,7 @@ DoVosPartitionList(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosServerSync(struct cmd_syndesc *as, char *arock)
+DoVosServerSync(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION } DoVosServerSync_parm_t;
     afs_status_t st = 0;
@@ -309,13 +309,14 @@ DoVosServerSync(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosFileServerAddressChange(struct cmd_syndesc *as, char *arock)
+DoVosFileServerAddressChange(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { OLDADDRESS,
 	NEWADDRESS
     } DoVosFileServerAddressChange_parm_t;
     afs_status_t st = 0;
-    int old_addr, new_addr;
+    int old_addr = 0;
+    int new_addr = 0;
 
     if (as->parms[OLDADDRESS].items) {
 	const char *addr = as->parms[OLDADDRESS].items->data;
@@ -334,11 +335,11 @@ DoVosFileServerAddressChange(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosFileServerAddressRemove(struct cmd_syndesc *as, char *arock)
+DoVosFileServerAddressRemove(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { ADDRESS } DoVosFileServerAddressRemove_parm_t;
     afs_status_t st = 0;
-    int address;
+    int address = 0;
 
     if (as->parms[ADDRESS].items) {
 	const char *addr = as->parms[ADDRESS].items->data;
@@ -363,7 +364,7 @@ Print_vos_fileServerEntry_p(vos_fileServerEntry_p serv, const char *prefix)
 }
 
 int
-DoVosFileServerList(struct cmd_syndesc *as, char *arock)
+DoVosFileServerList(struct cmd_syndesc *as, void *arock)
 {
     afs_status_t st = 0;
     void *iter;
@@ -414,7 +415,7 @@ Print_vos_serverTransactionStatus_p(vos_serverTransactionStatus_p tran,
 }
 
 int
-DoVosServerTransactionStatusList(struct cmd_syndesc *as, char *arock)
+DoVosServerTransactionStatusList(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER } DoVosServerTransactionStatusList_parm_t;
     afs_status_t st = 0;
@@ -522,13 +523,13 @@ Print_vos_vldbEntry_p(vos_vldbEntry_p entry, const char *prefix)
 }
 
 int
-DoVosVLDBGet(struct cmd_syndesc *as, char *arock)
+DoVosVLDBGet(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { VOLUME } DoVosVLDBGet_parm_t;
     afs_status_t st = 0;
     vos_vldbEntry_t entry;
     unsigned int volume_id;
-    const char *volume_name = NULL;
+    char *volume_name = NULL;
 
     if (as->parms[VOLUME].items) {
 	const char *volume = as->parms[VOLUME].items->data;
@@ -546,7 +547,7 @@ DoVosVLDBGet(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVLDBList(struct cmd_syndesc *as, char *arock)
+DoVosVLDBList(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION } DoVosVLDBList_parm_t;
     afs_status_t st = 0;
@@ -594,7 +595,7 @@ DoVosVLDBList(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVLDBEntryRemove(struct cmd_syndesc *as, char *arock)
+DoVosVLDBEntryRemove(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME } DoVosVLDBEntryRemove_parm_t;
     afs_status_t st = 0;
@@ -637,7 +638,7 @@ DoVosVLDBEntryRemove(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVLDBUnlock(struct cmd_syndesc *as, char *arock)
+DoVosVLDBUnlock(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION } DoVosVLDBUnlock_parm_t;
     afs_status_t st = 0;
@@ -671,11 +672,11 @@ DoVosVLDBUnlock(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVLDBEntryLock(struct cmd_syndesc *as, char *arock)
+DoVosVLDBEntryLock(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { VOLUME } DoVosVLDBEntryLoc_parm_tk;
     afs_status_t st = 0;
-    unsigned int volume_id;
+    unsigned int volume_id = 0;
 
     if (as->parms[VOLUME].items) {
 	const char *volume = as->parms[VOLUME].items->data;
@@ -690,11 +691,11 @@ DoVosVLDBEntryLock(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVLDBEntryUnlock(struct cmd_syndesc *as, char *arock)
+DoVosVLDBEntryUnlock(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { VOLUME } DoVosVLDBEntryUnlock_parm_t;
     afs_status_t st = 0;
-    unsigned int volume_id;
+    unsigned int volume_id = 0;
 
     if (as->parms[VOLUME].items) {
 	const char *volume = as->parms[VOLUME].items->data;
@@ -709,15 +710,15 @@ DoVosVLDBEntryUnlock(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVLDBReadOnlySiteCreate(struct cmd_syndesc *as, char *arock)
+DoVosVLDBReadOnlySiteCreate(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION,
 	VOLUME
     } DoVosVLDBReadOnlySiteCreate_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
-    unsigned int volume_id;
+    unsigned int partition_id = 0;
+    unsigned int volume_id = 0;
 
     if (as->parms[SERVER].items) {
 	if (!vos_ServerOpen
@@ -744,15 +745,15 @@ DoVosVLDBReadOnlySiteCreate(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVLDBReadOnlySiteDelete(struct cmd_syndesc *as, char *arock)
+DoVosVLDBReadOnlySiteDelete(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION,
 	VOLUME
     } DoVosVLDBReadOnlySiteDelete_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
-    unsigned int volume_id;
+    unsigned int partition_id = 0;
+    unsigned int volume_id = 0;
 
     if (as->parms[SERVER].items) {
 	if (!vos_ServerOpen
@@ -780,7 +781,7 @@ DoVosVLDBReadOnlySiteDelete(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVLDBSync(struct cmd_syndesc *as, char *arock)
+DoVosVLDBSync(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, FORCE } DoVosVLDBSync_parm_t;
     afs_status_t st = 0;
@@ -819,17 +820,17 @@ DoVosVLDBSync(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeCreate(struct cmd_syndesc *as, char *arock)
+DoVosVolumeCreate(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME,
 	QUOTA
     } DoVosVolumeCreate_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
+    unsigned int partition_id = 0;
     unsigned int volume_id;
-    const char *volume = NULL;
-    unsigned int quota;
+    char *volume = NULL;
+    unsigned int quota = 0;
 
     if (as->parms[SERVER].items) {
 	if (!vos_ServerOpen
@@ -864,13 +865,13 @@ DoVosVolumeCreate(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeDelete(struct cmd_syndesc *as, char *arock)
+DoVosVolumeDelete(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME } DoVosVolumeDelete_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
-    unsigned int volume_id;
+    unsigned int partition_id = 0;
+    unsigned int volume_id = 0;
 
     if (as->parms[SERVER].items) {
 	if (!vos_ServerOpen
@@ -898,12 +899,12 @@ DoVosVolumeDelete(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeRename(struct cmd_syndesc *as, char *arock)
+DoVosVolumeRename(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { OLDVOLUME, NEWVOLUME } DoVosVolumeRename_parm_t;
     afs_status_t st = 0;
-    unsigned int old_volume;
-    const char *new_volume;
+    unsigned int old_volume = 0;
+    char *new_volume = NULL;
 
     if (as->parms[OLDVOLUME].items) {
 	const char *volume = as->parms[OLDVOLUME].items->data;
@@ -922,7 +923,7 @@ DoVosVolumeRename(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeDump(struct cmd_syndesc *as, char *arock)
+DoVosVolumeDump(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME, STARTTIME,
 	DUMPFILE
@@ -932,9 +933,9 @@ DoVosVolumeDump(struct cmd_syndesc *as, char *arock)
     unsigned int partition_id;
     unsigned int *part_ptr = NULL;
     int have_server = 0;
-    unsigned int volume_id;
-    unsigned int start_time;
-    const char *dumpfile;
+    unsigned int volume_id = 0;
+    unsigned int start_time = 0;
+    const char *dumpfile = NULL;
 
     if (as->parms[SERVER].items) {
 	if (!vos_ServerOpen
@@ -978,18 +979,18 @@ DoVosVolumeDump(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeRestore(struct cmd_syndesc *as, char *arock)
+DoVosVolumeRestore(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, ID, VOLUME, DUMPFILE,
 	FULL
     } DoVosVolumeRestore_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
+    unsigned int partition_id = 0;
     unsigned int volume_id;
     unsigned int *vol_ptr = NULL;
-    const char *dumpfile;
-    const char *volume_name;
+    const char *dumpfile = NULL;
+    char *volume_name = NULL;
     vos_volumeRestoreType_t restore = VOS_RESTORE_INCREMENTAL;
 
     if (as->parms[SERVER].items) {
@@ -1032,16 +1033,16 @@ DoVosVolumeRestore(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeOnline(struct cmd_syndesc *as, char *arock)
+DoVosVolumeOnline(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME, SLEEP,
 	BUSY
     } DoVosVolumeOnline_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
-    unsigned int volume_id;
-    unsigned int sleep;
+    unsigned int partition_id = 0;
+    unsigned int volume_id = 0;
+    unsigned int sleep = 0;
     vos_volumeOnlineType_t type = VOS_ONLINE_OFFLINE;
 
     if (as->parms[SERVER].items) {
@@ -1080,13 +1081,13 @@ DoVosVolumeOnline(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeOffline(struct cmd_syndesc *as, char *arock)
+DoVosVolumeOffline(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME } DoVosVolumeOffline_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
-    unsigned int volume_id;
+    unsigned int partition_id = 0;
+    unsigned int volume_id = 0;
 
     if (as->parms[SERVER].items) {
 	if (!vos_ServerOpen
@@ -1337,13 +1338,13 @@ Print_vos_volumeEntry_p(vos_volumeEntry_p entry, const char *prefix)
 }
 
 int
-DoVosVolumeGet(struct cmd_syndesc *as, char *arock)
+DoVosVolumeGet(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME } DoVosVolumeGet_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
-    unsigned int volume_id;
+    unsigned int partition_id = 0;
+    unsigned int volume_id = 0;
     vos_volumeEntry_t entry;
 
     if (as->parms[SERVER].items) {
@@ -1374,13 +1375,13 @@ DoVosVolumeGet(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeList(struct cmd_syndesc *as, char *arock)
+DoVosVolumeList(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION } DoVosVolumeList_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
     void *iter = NULL;
-    unsigned int partition_id;
+    unsigned int partition_id = 0;
     vos_volumeEntry_t entry;
 
     if (as->parms[SERVER].items) {
@@ -1420,7 +1421,7 @@ DoVosVolumeList(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeMove(struct cmd_syndesc *as, char *arock)
+DoVosVolumeMove(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { VOLUME, FROMSERVER, FROMPARTITION, TOSERVER,
 	TOPARTITION
@@ -1428,9 +1429,9 @@ DoVosVolumeMove(struct cmd_syndesc *as, char *arock)
     afs_status_t st = 0;
     void *from_server = NULL;
     void *to_server = NULL;
-    unsigned int from_partition;
-    unsigned int to_partition;
-    unsigned int volume_id;
+    unsigned int from_partition = 0;
+    unsigned int to_partition = 0;
+    unsigned int volume_id = 0;
 
     if (as->parms[FROMSERVER].items) {
 	if (!vos_ServerOpen
@@ -1472,11 +1473,11 @@ DoVosVolumeMove(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeRelease(struct cmd_syndesc *as, char *arock)
+DoVosVolumeRelease(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { VOLUME, FORCE } DoVosVolumeRelease_parm_t;
     afs_status_t st = 0;
-    unsigned int volume_id;
+    unsigned int volume_id = 0;
     vos_force_t force = VOS_NORMAL;
 
     if (as->parms[VOLUME].items) {
@@ -1495,13 +1496,13 @@ DoVosVolumeRelease(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeZap(struct cmd_syndesc *as, char *arock)
+DoVosVolumeZap(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME, FORCE } DoVosVolumeZap_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
-    unsigned int volume_id;
+    unsigned int partition_id = 0;
+    unsigned int volume_id = 0;
     vos_force_t force = VOS_NORMAL;
 
     if (as->parms[SERVER].items) {
@@ -1534,11 +1535,10 @@ DoVosVolumeZap(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosPartitionNameToId(struct cmd_syndesc *as, char *arock)
+DoVosPartitionNameToId(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { PARTITION } DoVosPartitionNameToId_parm_t;
-    afs_status_t st = 0;
-    unsigned int partition_id;
+    unsigned int partition_id = 0;
 
     if (as->parms[PARTITION].items) {
 	partition_id =
@@ -1552,11 +1552,11 @@ DoVosPartitionNameToId(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosPartitionIdToName(struct cmd_syndesc *as, char *arock)
+DoVosPartitionIdToName(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { PARTITIONID } DoVosPartitionIdToName_parm_t;
     afs_status_t st = 0;
-    unsigned int partition_id;
+    unsigned int partition_id = 0;
     char partition[VOS_MAX_PARTITION_NAME_LEN];
 
     if (as->parms[PARTITIONID].items) {
@@ -1575,16 +1575,16 @@ DoVosPartitionIdToName(struct cmd_syndesc *as, char *arock)
 }
 
 int
-DoVosVolumeQuotaChange(struct cmd_syndesc *as, char *arock)
+DoVosVolumeQuotaChange(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME,
 	QUOTA
     } DoVosVolumeQuotaChange_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
-    unsigned int volume_id;
-    unsigned int quota;
+    unsigned int partition_id = 0;
+    unsigned int volume_id = 0;
+    unsigned int quota = 0;
 
     if (as->parms[SERVER].items) {
 	if (!vos_ServerOpen
@@ -1622,10 +1622,10 @@ DoVosVolumeQuotaChange(struct cmd_syndesc *as, char *arock)
 static afs_uint32
 GetServer(char *aname)
 {
-    register struct hostent *th;
+    struct hostent *th;
     afs_uint32 addr;
     int b1, b2, b3, b4;
-    register afs_int32 code;
+    afs_int32 code;
     char hostname[MAXHOSTCHARS];
 
     code = sscanf(aname, "%d.%d.%d.%d", &b1, &b2, &b3, &b4);
@@ -1639,7 +1639,7 @@ GetServer(char *aname)
 	memcpy(&addr, th->h_addr, sizeof(addr));
     }
 
-    if (addr == htonl(0x7f000001)) {	/* local host */
+    if (rx_IsLoopbackAddr(ntohl(addr))) {	/* local host */
 	code = gethostname(hostname, MAXHOSTCHARS);
 	if (code)
 	    return 0;
@@ -1671,7 +1671,7 @@ Print_vos_volintInfo(afs_uint32 server, afs_uint32 partition, volintInfo* pinfo,
     
     
     printf("%sname\t\t%s\n",prefix, pinfo->name);
-    printf("%sid\t\t%lu\n",prefix, pinfo->volid);
+    printf("%sid\t\t%lu\n",prefix, afs_printable_uint32_lu(pinfo->volid));
     printf("%sserv\t\t%s\t%s\n",prefix, address,hostname);
     printf("%spart\t\t%u\n", prefix,partition);
     
@@ -1686,9 +1686,12 @@ Print_vos_volintInfo(afs_uint32 server, afs_uint32 partition, volintInfo* pinfo,
 	printf("%sstatus\t\tUNATTACHABLE\n",prefix);
 	return;
     }
-    printf("%sbackupID\t%lu\n",prefix, pinfo->backupID);
-    printf("%sparentID\t%lu\n",prefix, pinfo->parentID);
-    printf("%scloneID\t%lu\n",prefix, pinfo->cloneID);
+    printf("%sbackupID\t%lu\n",prefix,
+	   afs_printable_uint32_lu(pinfo->backupID));
+    printf("%sparentID\t%lu\n",prefix,
+	   afs_printable_uint32_lu(pinfo->parentID));
+    printf("%scloneID\t%lu\n",prefix,
+	   afs_printable_uint32_lu(pinfo->cloneID));
     printf("%sinUse\t\t%s\n",prefix, pinfo->inUse ? "Y" : "N");
     printf("%sneedsSalvaged\t%s\n",prefix, pinfo->needsSalvaged ? "Y" : "N");
     /* 0xD3 is from afs/volume.h since I had trouble including the file */
@@ -1707,31 +1710,40 @@ Print_vos_volintInfo(afs_uint32 server, afs_uint32 partition, volintInfo* pinfo,
 	printf("%stype\t\t?\n",prefix);
 	break;
     }
-    printf("%screationDate\t%-9lu\n", prefix,pinfo->creationDate);
-    printf("%saccessDate\t%-9lu\n", prefix,pinfo->accessDate);
-    printf("%supdateDate\t%-9lu\n", prefix,pinfo->updateDate);
-    printf("%sbackupDate\t%-9lu\n", prefix,pinfo->backupDate);
-    printf("%scopyDate\t%-9lu\n", prefix,pinfo->copyDate);
+    printf("%screationDate\t%-9lu\n", prefix,
+	   afs_printable_uint32_lu(pinfo->creationDate));
+    printf("%saccessDate\t%-9lu\n", prefix,
+	   afs_printable_uint32_lu(pinfo->accessDate));
+    printf("%supdateDate\t%-9lu\n", prefix,
+	   afs_printable_uint32_lu(pinfo->updateDate));
+    printf("%sbackupDate\t%-9lu\n", prefix,
+	   afs_printable_uint32_lu(pinfo->backupDate));
+    printf("%scopyDate\t%-9lu\n", prefix,
+	   afs_printable_uint32_lu(pinfo->copyDate));
 	    
-    printf("%sflags\t\t%#lx\t(Optional)\n",prefix, pinfo->flags);
+    printf("%sflags\t\t%#lx\t(Optional)\n",prefix, afs_printable_int32_ld(pinfo->flags));
     printf("%sdiskused\t%u\n",prefix, pinfo->size);
     printf("%smaxquota\t%u\n",prefix, pinfo->maxquota);
-    printf("%sminquota\t%lu\t(Optional)\n",prefix, pinfo->spare0);
+    printf("%sminquota\t%lu\t(Optional)\n",prefix,
+	   afs_printable_uint32_lu(pinfo->spare0));
     printf("%sfilecount\t%u\n",prefix, pinfo->filecount);
     printf("%sdayUse\t\t%u\n",prefix, pinfo->dayUse);
-    printf("%sweekUse\t%lu\t(Optional)\n",prefix, pinfo->spare1);
-    printf("%svolUpdateCounter\t\t%lu\t(Optional)\n",prefix, pinfo->spare2);
-    printf("%sspare3\t\t%lu\t(Optional)\n",prefix, pinfo->spare3);
+    printf("%sweekUse\t%lu\t(Optional)\n",prefix,
+	   afs_printable_uint32_lu(pinfo->spare1));
+    printf("%svolUpdateCounter\t\t%lu\t(Optional)\n",prefix,
+           afs_printable_uint32_lu(pinfo->spare2));
+    printf("%sspare3\t\t%lu\t(Optional)\n",prefix,
+	   afs_printable_uint32_lu(pinfo->spare3));
 }
 
 int
-DoVosVolumeGet2(struct cmd_syndesc *as, char *arock)
+DoVosVolumeGet2(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME } DoVosVolumeGet_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    afs_uint32 partition_id;
-    afs_uint32 volume_id;
+    afs_uint32 partition_id = 0;
+    afs_uint32 volume_id = 0;
 
 	volintInfo info;
 	memset(&info, 0, sizeof(struct volintInfo));
@@ -1767,13 +1779,13 @@ DoVosVolumeGet2(struct cmd_syndesc *as, char *arock)
 
 
 int
-DoVos_ClearVolUpdateCounter(struct cmd_syndesc *as, char *arock)
+DoVos_ClearVolUpdateCounter(struct cmd_syndesc *as, void *arock)
 {
     typedef enum { SERVER, PARTITION, VOLUME } DoVosVolumeGet_parm_t;
     afs_status_t st = 0;
     void *vos_server = NULL;
-    unsigned int partition_id;
-    unsigned int volume_id;
+    unsigned int partition_id = 0;
+    unsigned int volume_id = 0;
 
     if (as->parms[SERVER].items) {
 	if (!vos_ServerOpen
@@ -1805,13 +1817,13 @@ SetupVosAdminCmd(void)
 {
     struct cmd_syndesc *ts;
 
-    ts = cmd_CreateSyntax("VosBackupVolumeCreate", DoVosBackupVolumeCreate, 0,
+    ts = cmd_CreateSyntax("VosBackupVolumeCreate", DoVosBackupVolumeCreate, NULL,
 			  "create a backup volume");
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_REQUIRED, "volume to back up");
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("VosBackupVolumeCreateMultiple",
-			  DoVosBackupVolumeCreateMultiple, 0,
+			  DoVosBackupVolumeCreateMultiple, NULL,
 			  "create a backup volume");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_OPTIONAL,
 		"server housing volumes to back up");
@@ -1823,7 +1835,7 @@ SetupVosAdminCmd(void)
 		"exclude volumes from backup that match prefix");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosPartitionGet", DoVosPartitionGet, 0,
+    ts = cmd_CreateSyntax("VosPartitionGet", DoVosPartitionGet, NULL,
 			  "get information about a partition");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server housing partition of interest");
@@ -1831,13 +1843,13 @@ SetupVosAdminCmd(void)
 		"partition to query");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosPartitionList", DoVosPartitionList, 0,
+    ts = cmd_CreateSyntax("VosPartitionList", DoVosPartitionList, NULL,
 			  "list information about all partitions at a server");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server housing partitions of interest");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosServerSync", DoVosServerSync, 0,
+    ts = cmd_CreateSyntax("VosServerSync", DoVosServerSync, NULL,
 			  "sync server with vldb");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED, "server to sync");
     cmd_AddParm(ts, "-partition", CMD_SINGLE, CMD_OPTIONAL,
@@ -1845,7 +1857,7 @@ SetupVosAdminCmd(void)
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("VosFileServerAddressChange",
-			  DoVosFileServerAddressChange, 0,
+			  DoVosFileServerAddressChange, NULL,
 			  "change a server's address in the vldb");
     cmd_AddParm(ts, "-oldaddress", CMD_SINGLE, CMD_REQUIRED,
 		"old address to change");
@@ -1853,29 +1865,29 @@ SetupVosAdminCmd(void)
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("VosFileServerAddressRemove",
-			  DoVosFileServerAddressRemove, 0,
+			  DoVosFileServerAddressRemove, NULL,
 			  "remove a server's address from the vldb");
     cmd_AddParm(ts, "-address", CMD_SINGLE, CMD_REQUIRED,
 		"address to remove");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosFileServerList", DoVosFileServerList, 0,
+    ts = cmd_CreateSyntax("VosFileServerList", DoVosFileServerList, NULL,
 			  "list the file servers in a cell");
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("VosServerTransactionStatusList",
-			  DoVosServerTransactionStatusList, 0,
+			  DoVosServerTransactionStatusList, NULL,
 			  "list the active transactions at a server");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED, "server to query");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVLDBGet", DoVosVLDBGet, 0,
+    ts = cmd_CreateSyntax("VosVLDBGet", DoVosVLDBGet, NULL,
 			  "get a vldb entry for a volume");
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_REQUIRED,
 		"volume to retrieve");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVLDBList", DoVosVLDBList, 0,
+    ts = cmd_CreateSyntax("VosVLDBList", DoVosVLDBList, NULL,
 			  "list a group of vldb entries");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_OPTIONAL,
 		"limit entries to a particular server");
@@ -1883,7 +1895,7 @@ SetupVosAdminCmd(void)
 		"limit entries to a particular partition");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVLDBEntryRemove", DoVosVLDBEntryRemove, 0,
+    ts = cmd_CreateSyntax("VosVLDBEntryRemove", DoVosVLDBEntryRemove, NULL,
 			  "remove vldb entries");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_OPTIONAL,
 		"limit entries to a particular server");
@@ -1892,7 +1904,7 @@ SetupVosAdminCmd(void)
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_OPTIONAL, "volume to remove");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVLDBUnlock", DoVosVLDBUnlock, 0,
+    ts = cmd_CreateSyntax("VosVLDBUnlock", DoVosVLDBUnlock, NULL,
 			  "unlock a group of vldb entries");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_OPTIONAL,
 		"limit entries to a particular server");
@@ -1900,18 +1912,18 @@ SetupVosAdminCmd(void)
 		"limit entries to a particular partition");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVLDBEntryLock", DoVosVLDBList, 0,
+    ts = cmd_CreateSyntax("VosVLDBEntryLock", DoVosVLDBList, NULL,
 			  "lock a single vldb entry");
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_REQUIRED, "volume to lock");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVLDBEntryUnlock", DoVosVLDBEntryUnlock, 0,
+    ts = cmd_CreateSyntax("VosVLDBEntryUnlock", DoVosVLDBEntryUnlock, NULL,
 			  "unlock a single vldb entry");
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_REQUIRED, "volume to unlock");
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("VosVLDBReadOnlySiteCreate",
-			  DoVosVLDBReadOnlySiteCreate, 0,
+			  DoVosVLDBReadOnlySiteCreate, NULL,
 			  "create a read only site");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where read only will be created");
@@ -1922,7 +1934,7 @@ SetupVosAdminCmd(void)
     SetupCommonCmdArgs(ts);
 
     ts = cmd_CreateSyntax("VosVLDBReadOnlySiteDelete",
-			  DoVosVLDBReadOnlySiteDelete, 0,
+			  DoVosVLDBReadOnlySiteDelete, NULL,
 			  "delete a read only site before initial replication");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_OPTIONAL,
 		"server where read only will be deleted");
@@ -1931,7 +1943,7 @@ SetupVosAdminCmd(void)
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_REQUIRED, "volume to delete");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVLDBSync", DoVosVLDBSync, 0,
+    ts = cmd_CreateSyntax("VosVLDBSync", DoVosVLDBSync, NULL,
 			  "sync vldb with server");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED, "server to sync");
     cmd_AddParm(ts, "-partition", CMD_SINGLE, CMD_OPTIONAL,
@@ -1939,7 +1951,7 @@ SetupVosAdminCmd(void)
     cmd_AddParm(ts, "-force", CMD_FLAG, CMD_OPTIONAL, "force sync to occur");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeCreate", DoVosVolumeCreate, 0,
+    ts = cmd_CreateSyntax("VosVolumeCreate", DoVosVolumeCreate, NULL,
 			  "create a read write volume");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where volume will be created");
@@ -1951,7 +1963,7 @@ SetupVosAdminCmd(void)
 		"size quota of new volume in 1kb units");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeDelete", DoVosVolumeDelete, 0,
+    ts = cmd_CreateSyntax("VosVolumeDelete", DoVosVolumeDelete, NULL,
 			  "delete a volume");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server where volume exists");
@@ -1960,13 +1972,13 @@ SetupVosAdminCmd(void)
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_REQUIRED, "volume to delete");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeRename", DoVosVolumeRename, 0,
+    ts = cmd_CreateSyntax("VosVolumeRename", DoVosVolumeRename, NULL,
 			  "rename a volume");
     cmd_AddParm(ts, "-oldname", CMD_SINGLE, CMD_REQUIRED, "old volume name");
     cmd_AddParm(ts, "-newname", CMD_SINGLE, CMD_REQUIRED, "new volume name");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeDump", DoVosVolumeDump, 0,
+    ts = cmd_CreateSyntax("VosVolumeDump", DoVosVolumeDump, NULL,
 			  "dump a volume to a file");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_OPTIONAL,
 		"dump volume at a particular server");
@@ -1979,7 +1991,7 @@ SetupVosAdminCmd(void)
 		"file to contain dump results");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeRestore", DoVosVolumeRestore, 0,
+    ts = cmd_CreateSyntax("VosVolumeRestore", DoVosVolumeRestore, NULL,
 			  "restore a volume from a dumpfile");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server that houses volume to restore");
@@ -1994,7 +2006,7 @@ SetupVosAdminCmd(void)
 		"does a full restore of volume");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeOnline", DoVosVolumeOnline, 0,
+    ts = cmd_CreateSyntax("VosVolumeOnline", DoVosVolumeOnline, NULL,
 			  "bring a volume online");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server that houses volume");
@@ -2006,7 +2018,7 @@ SetupVosAdminCmd(void)
     cmd_AddParm(ts, "-busy", CMD_FLAG, CMD_OPTIONAL, "mark volume busy");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeOffline", DoVosVolumeOffline, 0,
+    ts = cmd_CreateSyntax("VosVolumeOffline", DoVosVolumeOffline, NULL,
 			  "take a volume offline");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server that houses volume");
@@ -2016,7 +2028,7 @@ SetupVosAdminCmd(void)
 		"volume to bring offline");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeGet", DoVosVolumeGet, 0,
+    ts = cmd_CreateSyntax("VosVolumeGet", DoVosVolumeGet, NULL,
 			  "get a volume entry");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server that houses volume");
@@ -2026,7 +2038,7 @@ SetupVosAdminCmd(void)
 		"volume to retrieve");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeList", DoVosVolumeList, 0,
+    ts = cmd_CreateSyntax("VosVolumeList", DoVosVolumeList, NULL,
 			  "list a group of volumes");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"limit volumes to a particular server");
@@ -2034,7 +2046,7 @@ SetupVosAdminCmd(void)
 		"limit volumes to a particular partition");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeMove", DoVosVolumeMove, 0,
+    ts = cmd_CreateSyntax("VosVolumeMove", DoVosVolumeMove, NULL,
 			  "move a volume");
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_REQUIRED, "volume to move");
     cmd_AddParm(ts, "-fromserver", CMD_SINGLE, CMD_REQUIRED, "source server");
@@ -2046,7 +2058,7 @@ SetupVosAdminCmd(void)
 		"destination partition");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeRelease", DoVosVolumeRelease, 0,
+    ts = cmd_CreateSyntax("VosVolumeRelease", DoVosVolumeRelease, NULL,
 			  "release updates to read only");
     cmd_AddParm(ts, "-volume", CMD_SINGLE, CMD_REQUIRED,
 		"volume to replicate");
@@ -2054,7 +2066,7 @@ SetupVosAdminCmd(void)
 		"force release to occur");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeZap", DoVosVolumeZap, 0, "zap a volume");
+    ts = cmd_CreateSyntax("VosVolumeZap", DoVosVolumeZap, NULL, "zap a volume");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server that houses the volume to zap");
     cmd_AddParm(ts, "-partition", CMD_SINGLE, CMD_REQUIRED,
@@ -2063,18 +2075,18 @@ SetupVosAdminCmd(void)
     cmd_AddParm(ts, "-force", CMD_FLAG, CMD_OPTIONAL, "force zap");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosPartitionNameToId", DoVosPartitionNameToId, 0,
+    ts = cmd_CreateSyntax("VosPartitionNameToId", DoVosPartitionNameToId, NULL,
 			  "convert a partition name to a number");
     cmd_AddParm(ts, "-partition", CMD_SINGLE, CMD_REQUIRED,
 		"partition to convert");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosPartitionIdToName", DoVosPartitionIdToName, 0,
+    ts = cmd_CreateSyntax("VosPartitionIdToName", DoVosPartitionIdToName, NULL,
 			  "convert a number to a partition");
     cmd_AddParm(ts, "-id", CMD_SINGLE, CMD_REQUIRED, "number to convert");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeQuotaChange", DoVosVolumeQuotaChange, 0,
+    ts = cmd_CreateSyntax("VosVolumeQuotaChange", DoVosVolumeQuotaChange, NULL,
 			  "change the quota for a partition");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server that houses the volume");
@@ -2085,7 +2097,7 @@ SetupVosAdminCmd(void)
 		"new quota in 1kb units");
     SetupCommonCmdArgs(ts);
 
-    ts = cmd_CreateSyntax("VosVolumeGet2", DoVosVolumeGet2, 0,
+    ts = cmd_CreateSyntax("VosVolumeGet2", DoVosVolumeGet2, NULL,
 			  "get a volume entry");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server that houses volume");
@@ -2095,7 +2107,7 @@ SetupVosAdminCmd(void)
 		"volume to retrieve");
     SetupCommonCmdArgs(ts);
     
-    ts = cmd_CreateSyntax("ClearVolUpdateCounter", DoVos_ClearVolUpdateCounter, 0,
+    ts = cmd_CreateSyntax("ClearVolUpdateCounter", DoVos_ClearVolUpdateCounter, NULL,
 			  "clear volUpdateCounter");
     cmd_AddParm(ts, "-server", CMD_SINGLE, CMD_REQUIRED,
 		"server that houses volume");
